@@ -1,7 +1,7 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2003,2004,2005,2006 Bernhard R. Link
+ *  Copyright (C) 2003,2004,2005,2006,2007 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as 
+ *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
@@ -45,6 +45,9 @@
 #include "checkindeb.h"
 #include "checkin.h"
 #include "uploaderslist.h"
+#include "log.h"
+#include "dpkgversions.h"
+#include "changes.h"
 
 extern int verbose;
 
@@ -68,11 +71,6 @@ extern int verbose;
  *
  */
 
-typedef	enum { fe_UNKNOWN=0,fe_DEB,fe_UDEB,fe_DSC,fe_DIFF,fe_ORIG,fe_TAR} filetype;
-
-#define FE_BINARY(ft) ( (ft) == fe_DEB || (ft) == fe_UDEB )
-#define FE_SOURCE(ft) ( (ft) == fe_DIFF || (ft) == fe_ORIG || (ft) == fe_TAR || (ft) == fe_DSC || (ft) == fe_UNKNOWN)
-
 struct fileentry {
 	struct fileentry *next;
 	char *basename;
@@ -97,7 +95,7 @@ struct fileentry {
 
 struct changes {
 	/* Things read by changes_read: */
-	char *source, *version;
+	char *source, *sourceversion, *changesversion;
 	struct strlist distributions,
 		       architectures,
 		       binaries;
@@ -115,6 +113,8 @@ struct changes {
 	const char *firstcomponent;
 	/* the directory the .changes file resides in */
 	char *incomingdirectory;
+	/* the Version: and the version in Source: differ */
+	bool_t isbinnmu;
 };
 
 static void freeentries(/*@only@*/struct fileentry *entry) {
@@ -142,7 +142,8 @@ static void freeentries(/*@only@*/struct fileentry *entry) {
 static void changes_free(/*@only@*/struct changes *changes) {
 	if( changes != NULL ) {
 		free(changes->source);
-		free(changes->version);
+		free(changes->sourceversion);
+		free(changes->changesversion);
 		strlist_done(&changes->architectures);
 		strlist_done(&changes->binaries);
 		freeentries(changes->files);
@@ -160,163 +161,37 @@ static void changes_free(/*@only@*/struct changes *changes) {
 
 static retvalue newentry(struct fileentry **entry,const char *fileline,const char *packagetypeonly,const char *forcearchitecture, const char *sourcename) {
 	struct fileentry *e;
-	const char *p,*md5start,*md5end;
-	const char *sizestart,*sizeend;
-	const char *sectionstart,*sectionend;
-	const char *priostart,*prioend;
-	const char *filestart,*nameend,*fileend;
-	const char *archstart,*archend;
-	const char *versionstart,*typestart;
-	filetype type;
+	retvalue r;
 
-	p = fileline;
-	while( *p !='\0' && xisspace(*p) )
-		p++;
-	md5start = p;
-	while( *p !='\0' && !xisspace(*p) )
-		p++;
-	md5end = p;
-	while( *p !='\0' && xisspace(*p) )
-		p++;
-	sizestart = p;
-	while( *p !='\0' && !xisspace(*p) )
-		p++;
-	sizeend = p;
-	while( *p !='\0' && xisspace(*p) )
-		p++;
-	sectionstart = p;
-	while( *p !='\0' && !xisspace(*p) )
-		p++;
-	sectionend = p;
-	while( *p !='\0' && xisspace(*p) )
-		p++;
-	priostart = p;
-	while( *p !='\0' && !xisspace(*p) )
-		p++;
-	prioend = p;
-	while( *p !='\0' && xisspace(*p) )
-		p++;
-	filestart = p;
-	while( *p !='\0' && !xisspace(*p) )
-		p++;
-	fileend = p;
-	while( *p !='\0' && xisspace(*p) )
-		p++;
-	if( *p != '\0' ) {
-		fprintf(stderr,"Unexpected sixth argument in '%s'!\n",fileline);
-		return RET_ERROR;
-	}
-	if( *md5start == '\0' || *sizestart == '\0' || *sectionstart == '\0'
-			|| *priostart == '\0' || *filestart == '\0' ) {
-		fprintf(stderr,"Not five arguments in '%s'!\n",fileline);
-		return RET_ERROR;
-	}
-
-	p = filestart;
-	while( *p != '\0' && *p != '_' && !xisspace(*p) )
-		p++;
-	if( *p != '_' ) {
-		if( *p == '\0' )
-			fprintf(stderr,"No underscore in filename in '%s'!",fileline);
-		else
-			fprintf(stderr,"Unexpected character '%c' in filename in '%s'\n!",*p,fileline);
-		return RET_ERROR;
-	}
-	nameend = p;
-	p++;
-	versionstart = p;
-	// We cannot say where the version ends and the filename starts,
-	// but as the packagetypes would be valid part of the version, too,
-	// this check gets the broken things. 
-	names_overversion(&p,TRUE);
-	if( *p != '\0' && *p != '_' ) {
-		fprintf(stderr,"Unexpected character '%c' in filename within '%s'!\n",*p,fileline);
-		return RET_ERROR;
-	}
-	if( *p == '_' ) {
-		/* Things having a underscole will have an architecture
-		 * and be either .deb or .udeb */
-		p++;
-		archstart = p;
-		while( *p !='\0' && *p != '.' )
-			p++;
-		if( *p != '.' ) {
-			fprintf(stderr,"Expect something of the form name_version_arch.[u]deb but got '%s'!\n",filestart);
-			return RET_ERROR;
-		}
-		archend = p;
-		p++;
-		typestart = p;
-		while( *p !='\0' && !xisspace(*p) )
-			p++;
-		if( p-typestart == 3 && strncmp(typestart,"deb",3) == 0 )
-			type = fe_DEB;
-		else if( p-typestart == 4 && strncmp(typestart,"udeb",4) == 0 )
-			type = fe_UDEB;
-		else {
-			fprintf(stderr,"'%s' looks neighter like .deb nor like .udeb!\n",filestart);
-			return RET_ERROR;
-		}
-		if( strncmp(archstart,"source",6) == 0 ) {
-			fprintf(stderr,"How can a .[u]deb be architecture 'source'?('%s')\n",filestart);
-			return RET_ERROR;
-		}
-	} else {
-		/* this looks like some source-package, we will have
-		 * to look for the packagetype ourself... */
-		while( *p !='\0' && !xisspace(*p) ) {
-			p++;
-		}
-		if( p-versionstart > 12 && strncmp(p-12,".orig.tar.gz",12) == 0 )
-			type = fe_ORIG;
-		else if( p-versionstart > 7 && strncmp(p-7,".tar.gz",7) == 0 )
-			type = fe_TAR;
-		else if( p-versionstart > 8 && strncmp(p-8,".diff.gz",8) == 0 )
-			type = fe_DIFF;
-		else if( p-versionstart > 4 && strncmp(p-4,".dsc",4) == 0 )
-			type = fe_DSC;
-		else if( p-versionstart > 13 && strncmp(p-13,".orig.tar.bz2",13) == 0 )
-			type = fe_ORIG;
-		else if( p-versionstart > 8 && strncmp(p-8,".tar.bz2",8) == 0 )
-			type = fe_TAR;
-		else if( p-versionstart > 9 && strncmp(p-9,".diff.bz2",9) == 0 )
-			type = fe_DIFF;
-		else {
-			type = fe_UNKNOWN;
-			fprintf(stderr,"Unknown filetype: '%s', assuming to be source format...\n",fileline);
-		}
-		archstart = "source";
-		archend = archstart + 6;
-		if( strncmp(filestart,sourcename,nameend-filestart) != 0 ) {
-			fprintf(stderr,"Warning: Strange file '%s'!\nLooks like source but does not start with '%s_' as I would have guessed!\nI hope you know what you do.\n",filestart,sourcename);
-		}
-	}
-	if( FE_SOURCE(type) && packagetypeonly != NULL && strcmp(packagetypeonly,"dsc")!=0)
-		return RET_NOTHING;
-	if( type == fe_DEB && packagetypeonly != NULL && strcmp(packagetypeonly,"deb")!=0)
-		return RET_NOTHING;
-	if( type == fe_UDEB && packagetypeonly != NULL && strcmp(packagetypeonly,"udeb")!=0)
-		return RET_NOTHING;
-
-	/* now copy all those parts into the structure */
 	e = calloc(1,sizeof(struct fileentry));
 	if( e == NULL )
 		return RET_ERROR_OOM;
-	e->md5sum = names_concatmd5sumandsize(md5start,md5end,sizestart,sizeend);
-	e->section = strndup(sectionstart,sectionend-sectionstart);
-	e->priority = strndup(priostart,prioend-priostart);
-	e->basename = strndup(filestart,fileend-filestart);
-	e->architecture = strndup(archstart,archend-archstart);
-	e->name = strndup(filestart,nameend-filestart);
-	e->type = type;
 
-	if( e->basename == NULL || e->md5sum == NULL || e->section == NULL || 
-	    e->priority == NULL || e->architecture == NULL || e->name == NULL ) {
-		freeentries(e);
-		return RET_ERROR_OOM;
+	r = changes_parsefileline(fileline, &e->type, &e->basename, &e->md5sum,
+			&e->section, &e->priority, &e->architecture, &e->name);
+	if( RET_WAS_ERROR(r) ) {
+		free(e);
+		return r;
 	}
+	assert( RET_IS_OK(r) );
+	if( FE_SOURCE(e->type) && packagetypeonly != NULL && strcmp(packagetypeonly,"dsc")!=0) {
+		freeentries(e);
+		return RET_NOTHING;
+	}
+	if( e->type == fe_DEB && packagetypeonly != NULL && strcmp(packagetypeonly,"deb")!=0) {
+		freeentries(e);
+		return RET_NOTHING;
+	}
+	if( e->type == fe_UDEB && packagetypeonly != NULL && strcmp(packagetypeonly,"udeb")!=0) {
+		freeentries(e);
+		return RET_NOTHING;
+	}
+	if( strcmp(e->architecture, "source") == 0 && strcmp(e->name, sourcename) != 0 ) {
+		fprintf(stderr,"Warning: Strange file '%s'!\nLooks like source but does not start with '%s_' as I would have guessed!\nI hope you know what you do.\n",e->basename,sourcename);
+	}
+
 	if( forcearchitecture != NULL ) {
-		if( strcmp(forcearchitecture,"source") != 0 && 
+		if( strcmp(forcearchitecture,"source") != 0 &&
 				strcmp(e->architecture,"all") == 0 ) {
 			if( verbose > 2 )
 				fprintf(stderr,"Placing '%s' only in architecture '%s' as requested.\n",e->basename,forcearchitecture);
@@ -372,7 +247,7 @@ static retvalue check(const char *filename,struct changes *changes,const char *f
 			return RET_OK;
 		} else {
 			return RET_ERROR;
-		} 
+		}
 	}
 	return r;
 }
@@ -382,6 +257,7 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 	struct changes *c;
 	struct strlist filelines;
 	bool_t broken;
+	int versioncmp;
 
 #define E(err) { \
 		if( r == RET_NOTHING ) { \
@@ -399,14 +275,15 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 			return r; \
 		} \
 	}
-			
-		
+
+
 	c = calloc(1,sizeof(struct changes));
 	if( c == NULL )
 		return RET_ERROR_OOM;
-	r = signature_readsignedchunk(filename,&c->control,&c->fingerprints, NULL, &broken);
+	r = signature_readsignedchunk(filename, filename,
+			&c->control, &c->fingerprints, NULL, &broken);
 	R;
-	if( broken && !IGNORING_(brokensignatures, 
+	if( broken && !IGNORING_(brokensignatures,
 "'%s' contains only broken signatures.\n"
 "This most likely means the file was damaged (or edited improperly)\n",
 				filename) ) {
@@ -417,18 +294,35 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 	R;
 	r = check(filename,c,"Date");
 	R;
-	r = chunk_getname(c->control,"Source",&c->source,FALSE);
+	r = chunk_getnameandversion(c->control,"Source",&c->source,&c->sourceversion);
 	E("Missing 'Source' field");
 	r = propersourcename(c->source);
 	R;
+	if( c->sourceversion != NULL ) {
+		r = properversion(c->sourceversion);
+		R;
+	}
 	r = chunk_getwordlist(c->control,"Binary",&c->binaries);
 	E("Missing 'Binary' field");
 	r = chunk_getwordlist(c->control,"Architecture",&c->architectures);
 	E("Missing 'Architecture' field");
-	r = chunk_getvalue(c->control,"Version",&c->version);
+	r = chunk_getvalue(c->control,"Version",&c->changesversion);
 	E("Missing 'Version' field");
-	r = properversion(c->version);
+	r = properversion(c->changesversion);
 	E("Malforce Version number");
+	if( c->sourceversion == NULL ) {
+		c->sourceversion = strdup(c->changesversion);
+		if( c->sourceversion == NULL ) {
+			changes_free(c);
+			return RET_ERROR_OOM;
+		}
+		c->isbinnmu = FALSE;
+	} else {
+		r = dpkgversions_cmp(c->sourceversion, c->changesversion,
+				&versioncmp);
+		E("Error comparing versions. (That should have been caught earlier, why now?)");
+		c->isbinnmu = versioncmp != 0;
+	}
 	r = chunk_getwordlist(c->control,"Distribution",&c->distributions);
 	E("Missing 'Distribution' field");
 	r = check(filename,c,"Urgency");
@@ -453,7 +347,7 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 #undef R
 }
 
-static retvalue changes_fixfields(const struct distribution *distribution,const char *filename,struct changes *changes,/*@null@*/const char *forcecomponent,/*@null@*/const char *forcesection,/*@null@*/const char *forcepriority,const struct alloverrides *ao) {
+static retvalue changes_fixfields(const struct distribution *distribution,const char *filename,struct changes *changes,/*@null@*/const char *forcecomponent,/*@null@*/const char *forcesection,/*@null@*/const char *forcepriority) {
 	struct fileentry *e;
 	retvalue r;
 
@@ -466,17 +360,17 @@ static retvalue changes_fixfields(const struct distribution *distribution,const
 		fprintf(stderr,"No files given in '%s'!\n",filename);
 		return RET_ERROR;
 	}
-	
+
 	while( e != NULL ) {
 		const struct overrideinfo *oinfo = NULL;
 		const char *force = NULL;
 		if( forcesection == NULL || forcepriority == NULL ) {
 			oinfo = override_search(
-			FE_BINARY(e->type)?(e->type==fe_UDEB?ao->udeb:ao->deb):ao->dsc,
+			FE_BINARY(e->type)?(e->type==fe_UDEB?distribution->overrides.udeb:distribution->overrides.deb):distribution->overrides.dsc,
 					e->name);
 		}
-		
-		if( forcesection != NULL ) 
+
+		if( forcesection != NULL )
 			force = forcesection;
 		else
 			force = override_get(oinfo,SECTION_FIELDNAME);
@@ -495,7 +389,7 @@ static retvalue changes_fixfields(const struct distribution *distribution,const
 			return RET_ERROR;
 		}
 		if( strcmp(e->section,"-") == 0 ) {
-			fprintf(stderr,"No section specified for '%s'!\n",filename);
+			fprintf(stderr,"No section specified for '%s' in '%s'!\n", e->basename, filename);
 			return RET_ERROR;
 		}
 		if( forcepriority != NULL )
@@ -547,7 +441,7 @@ static retvalue changes_fixfields(const struct distribution *distribution,const
 			if( RET_WAS_ERROR(r) )
 				return r;
 			// Let's just check here, perhaps
-			if( e->type == fe_UDEB && 
+			if( e->type == fe_UDEB &&
 					!strlist_in(&distribution->udebcomponents,e->component)) {
 				fprintf(stderr,"Cannot put file '%s' into component '%s', as it is not listed in UDebComponents!\n",e->basename,e->component);
 				return RET_ERROR;
@@ -602,7 +496,7 @@ static retvalue changes_check(const char *filename,struct changes *changes,/*@nu
 	struct fileentry *e;
 	retvalue r = RET_OK;
 	bool_t havedsc=FALSE, haveorig=FALSE, havetar=FALSE, havediff=FALSE;
-	
+
 	/* First check for each given architecture, if it has files: */
 	if( forcearchitecture != NULL ) {
 		if( !strlist_in(&changes->architectures,forcearchitecture) ){
@@ -657,7 +551,7 @@ static retvalue changes_check(const char *filename,struct changes *changes,/*@nu
 				return RET_ERROR;
 			}
 			havedsc = TRUE;
-			calculatedname = calc_source_basename(changes->source,changes->version);
+			calculatedname = calc_source_basename(changes->source,changes->sourceversion);
 			if( calculatedname == NULL )
 				return RET_ERROR_OOM;
 			if( strcmp(calculatedname,e->basename) != 0 ) {
@@ -698,9 +592,9 @@ static retvalue changes_check(const char *filename,struct changes *changes,/*@nu
 		return RET_ERROR;
 	}
 	if( strlist_in(&changes->architectures,"source") && !havedsc &&
-			( forcearchitecture == NULL 
+			( forcearchitecture == NULL
 			  || strcmp(forcearchitecture,"source") == 0 ) &&
-			( packagetypeonly == NULL 
+			( packagetypeonly == NULL
 			  || strcmp(packagetypeonly,"dsc") == 0 )
 			) {
 		fprintf(stderr,"I don't know what to do with a source-upload not containing a .dsc in '%s'!\n",filename);
@@ -833,17 +727,16 @@ static retvalue changes_deleteleftoverfiles(struct changes *changes,int delete)
 					fullorigfilename, e, strerror(e));
 			r = RET_ERRNO(e);
 			RET_UPDATE(result,r);
-		} 
+		}
 		free(fullorigfilename);
 	}
 
 	return result;
 }
 
-static retvalue changes_checkpkgs(filesdb filesdb,struct distribution *distribution,struct changes *changes,const struct alloverrides *ao, const char *sourcedirectory) {
+static retvalue changes_checkpkgs(filesdb filesdb,struct distribution *distribution,struct changes *changes, const char *sourcedirectory) {
 	struct fileentry *e;
 	retvalue r;
-	bool_t somethingwasmissed = FALSE;
 
 	r = RET_NOTHING;
 
@@ -864,11 +757,9 @@ static retvalue changes_checkpkgs(filesdb filesdb,struct distribution *distribut
 				"deb",
 				distribution,fullfilename,
 				e->filekey,e->md5sum,
-				ao->deb,D_INPLACE,FALSE,
+				D_INPLACE,FALSE,
 				&changes->binaries,
-				changes->source,changes->version);
-			if( r == RET_NOTHING )
-				somethingwasmissed = TRUE;
+				changes->source,changes->sourceversion);
 		} else if( e->type == fe_UDEB ) {
 			r = deb_prepare(&e->pkg.deb,filesdb,
 				e->component,e->architecture,
@@ -876,42 +767,47 @@ static retvalue changes_checkpkgs(filesdb filesdb,struct distribution *distribut
 				"udeb",
 				distribution,fullfilename,
 				e->filekey,e->md5sum,
-				ao->udeb,D_INPLACE,FALSE,
+				D_INPLACE,FALSE,
 				&changes->binaries,
-				changes->source,changes->version);
-			if( r == RET_NOTHING )
-				somethingwasmissed = TRUE;
+				changes->source,changes->sourceversion);
 		} else if( e->type == fe_DSC ) {
-			assert(changes->srccomponent!=NULL);
-			assert(changes->srcdirectory!=NULL);
-			r = dsc_prepare(&e->pkg.dsc,filesdb,
-				changes->srccomponent,e->section,e->priority,
-				distribution,sourcedirectory,fullfilename,
-				e->filekey,e->basename,
-				changes->srcdirectory,e->md5sum,
-				ao->dsc,D_INPLACE,
-				changes->source,changes->version);
-			if( r == RET_NOTHING )
-				somethingwasmissed = TRUE;
+			if( !changes->isbinnmu || IGNORING_(dscinbinnmu,
+"File '%s' looks like a source package, but this .changes looks like a binNMU\n"
+"(as '%s' (from Source:) and '%s' (From Version:) differ.)\n",
+				e->filekey, changes->sourceversion,
+				changes->changesversion) ) {
+
+				assert(changes->srccomponent!=NULL);
+				assert(changes->srcdirectory!=NULL);
+				r = dsc_prepare(&e->pkg.dsc,filesdb,
+						changes->srccomponent,e->section,e->priority,
+						distribution,sourcedirectory,fullfilename,
+						e->filekey,e->basename,
+						changes->srcdirectory,e->md5sum,
+						D_INPLACE,
+						changes->source,changes->sourceversion);
+			} else
+				r = RET_ERROR;
 		}
-		
+
 		free(fullfilename);
 		if( RET_WAS_ERROR(r) )
 			break;
 		e = e->next;
 	}
 
-	if( RET_IS_OK(r) && somethingwasmissed ) {
-		return RET_NOTHING;
-	}
 	return r;
 }
-static retvalue changes_includepkgs(const char *dbdir,references refs,struct distribution *distribution,struct changes *changes,/*@null@*/struct strlist *dereferencedfilekeys, /*@null@*/struct trackingdata *trackingdata) {
+static retvalue changes_includepkgs(const char *dbdir,references refs,struct distribution *distribution,struct changes *changes,/*@null@*/struct strlist *dereferencedfilekeys, /*@null@*/struct trackingdata *trackingdata,bool_t *missed_p) {
 	struct fileentry *e;
-	retvalue r;
-	bool_t somethingwasmissed = FALSE;
+	retvalue result,r;
 
-	r = RET_NOTHING;
+	*missed_p = FALSE;
+	r = distribution_prepareforwriting(distribution);
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	result = RET_NOTHING;
 
 	e = changes->files;
 	while( e != NULL ) {
@@ -927,29 +823,29 @@ static retvalue changes_includepkgs(const char *dbdir,references refs,struct dis
 				e->architecture,"deb",
 				distribution,dereferencedfilekeys,trackingdata);
 			if( r == RET_NOTHING )
-				somethingwasmissed = TRUE;
+				*missed_p = TRUE;
 		} else if( e->type == fe_UDEB ) {
 			r = deb_addprepared(e->pkg.deb,dbdir,refs,
 				e->architecture,"udeb",
 				distribution,dereferencedfilekeys,trackingdata);
 			if( r == RET_NOTHING )
-				somethingwasmissed = TRUE;
+				*missed_p = TRUE;
 		} else if( e->type == fe_DSC ) {
 			r = dsc_addprepared(e->pkg.dsc,dbdir,refs,
 				distribution,dereferencedfilekeys,trackingdata);
 			if( r == RET_NOTHING )
-				somethingwasmissed = TRUE;
+				*missed_p = TRUE;
 		}
-		
+		RET_UPDATE(result, r);
+
 		if( RET_WAS_ERROR(r) )
 			break;
 		e = e->next;
 	}
 
-	if( RET_IS_OK(r) && somethingwasmissed ) {
-		return RET_NOTHING;
-	}
-	return r;
+	logger_wait();
+
+	return result;
 }
 
 static bool_t permissionssuffice(UNUSED(struct changes *changes),
@@ -960,31 +856,37 @@ static bool_t permissionssuffice(UNUSED(struct changes *changes),
 /* insert the given .changes into the mirror in the <distribution>
  * if forcecomponent, forcesection or forcepriority is NULL
  * get it from the files or try to guess it. */
-retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,filesdb filesdb,const char *packagetypeonly,const char *forcecomponent,const char *forcearchitecture,const char *forcesection,const char *forcepriority,struct distribution *distribution,struct uploaders *uploaders, const struct alloverrides *ao,const char *changesfilename,int delete,struct strlist *dereferencedfilekeys) {
+retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,filesdb filesdb,const char *packagetypeonly,const char *forcecomponent,const char *forcearchitecture,const char *forcesection,const char *forcepriority,struct distribution *distribution,const char *changesfilename,int delete,struct strlist *dereferencedfilekeys) {
 	retvalue result,r;
 	struct changes *changes;
 	struct trackingdata trackingdata;
 	char *directory;
+	bool_t somethingwasmissed;
 
 	r = changes_read(changesfilename,&changes,packagetypeonly,forcearchitecture);
 	if( RET_WAS_ERROR(r) )
 		return r;
 
-	if( (distribution->suite == NULL || 
+	if( (distribution->suite == NULL ||
 		!strlist_in(&changes->distributions,distribution->suite)) &&
-	    !strlist_in(&changes->distributions,distribution->codename) ) {
+	    !strlist_in(&changes->distributions,distribution->codename) &&
+	    !strlist_intersects(&changes->distributions,
+	                       &distribution->alsoaccept) ) {
 		if( !IGNORING("Ignoring","To ignore",wrongdistribution,".changes put in a distribution not listed within it!\n") ) {
 			changes_free(changes);
 			return RET_ERROR;
 		}
 	}
 
-	if( uploaders != NULL ) {
+	/* make sure caller has called distribution_loaduploaders */
+	assert( distribution->uploaders == NULL || distribution->uploaderslist != NULL );
+	if( distribution->uploaderslist != NULL ) {
 		const struct uploadpermissions *permissions;
 		int i;
 
 		if( changes->fingerprints.count == 0 ) {
-			r = uploaders_unsignedpermissions(uploaders, &permissions);
+			r = uploaders_unsignedpermissions(distribution->uploaderslist,
+					&permissions);
 			assert( r != RET_NOTHING );
 			if( RET_WAS_ERROR(r) ) {
 				changes_free(changes);
@@ -995,7 +897,8 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 		}
 		for( i = 0; i < changes->fingerprints.count ; i++ ) {
 			const char *fingerprint = changes->fingerprints.values[i];
-			r = uploaders_permissions(uploaders, fingerprint, &permissions);
+			r = uploaders_permissions(distribution->uploaderslist,
+					fingerprint, &permissions);
 			assert( r != RET_NOTHING );
 			if( RET_WAS_ERROR(r) ) {
 				changes_free(changes);
@@ -1024,7 +927,7 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 
 
 	/* look for component, section and priority to be correct or guess them*/
-	r = changes_fixfields(distribution,changesfilename,changes,forcecomponent,forcesection,forcepriority,ao);
+	r = changes_fixfields(distribution,changesfilename,changes,forcecomponent,forcesection,forcepriority);
 
 	/* do some tests if values are sensible */
 	if( !RET_WAS_ERROR(r) )
@@ -1044,7 +947,7 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 		r = changes_includefiles(filesdb,changes,delete);
 
 	if( !RET_WAS_ERROR(r) )
-		r = changes_checkpkgs(filesdb,distribution,changes,ao,directory);
+		r = changes_checkpkgs(filesdb,distribution,changes,directory);
 
 	free(directory);
 
@@ -1055,18 +958,19 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 	}
 
 	if( tracks != NULL ) {
-		r = trackingdata_summon(tracks,changes->source,changes->version,&trackingdata);
+		r = trackingdata_summon(tracks,changes->source,changes->sourceversion,&trackingdata);
 		if( RET_WAS_ERROR(r) ) {
 			changes_free(changes);
 			return r;
 		}
 		if( distribution->trackingoptions.includechanges ) {
-			const char *basename;
+			char *basename;
 			assert( changes->srcdirectory != NULL );
 
-			basename = dirs_basename(changesfilename);
-			changes->changesfilekey = 
+			basename = calc_changes_basename(changes->source, changes->changesversion, &changes->architectures);
+			changes->changesfilekey =
 				calc_dirconcat(changes->srcdirectory,basename);
+			free(basename);
 			if( changes->changesfilekey == NULL ) {
 				changes_free(changes);
 				trackingdata_done(&trackingdata);
@@ -1087,7 +991,8 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 		}
 	}
 	if( interrupted() ) {
-		trackingdata_done(&trackingdata);
+		if( tracks != NULL )
+			trackingdata_done(&trackingdata);
 		changes_free(changes);
 		return RET_ERROR_INTERUPTED;
 	}
@@ -1095,7 +1000,7 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 	/* add the source and binary packages in the given distribution */
 	result = changes_includepkgs(dbdir,refs,
 		distribution,changes,dereferencedfilekeys,
-		(tracks!=NULL)?&trackingdata:NULL);
+		(tracks!=NULL)?&trackingdata:NULL, &somethingwasmissed);
 
 	if( RET_WAS_ERROR(result) ) {
 		if( tracks != NULL ) {
@@ -1126,15 +1031,26 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 		}
 	}
 
+	/* if something was included, call --changes notify scripts */
+	if( RET_IS_OK(result) ) {
+		assert( logger_isprepared(distribution->logger) );
+		logger_logchanges(distribution->logger, distribution->codename,
+			changes->source, changes->changesversion, changes->control,
+			changesfilename, changes->changesfilekey);
+	}
+	/* wait for notify scripts (including those for the packages)
+	 * before deleting the .changes */
+	logger_wait();
+
 	if( (delete >= D_MOVE && changes->changesfilekey != NULL) ||
 			delete >= D_DELETE ) {
-		if( result == RET_NOTHING && delete < D_DELETE ) {
+		if( somethingwasmissed && delete < D_DELETE ) {
 			if( verbose >= 0 ) {
 				fprintf(stderr,"Not deleting '%s' as no package was added or some package was missed.\n(Use --delete --delete to delete anyway in such cases)\n",changesfilename);
 			}
 		} else {
 			if( verbose >= 5 ) {
-				fprintf(stderr,"Deleting '%s'.\n",changesfilename);
+				printf("Deleting '%s'.\n",changesfilename);
 			}
 			if( unlink(changesfilename) != 0 ) {
 				fprintf(stderr,"Error deleting '%s': %m\n",changesfilename);
