@@ -11,9 +11,6 @@
 #ifndef REPREPRO_FILES_H
 #include "files.h"
 #endif
-#ifndef REPREPRO_OVERRIDE_H
-#include "override.h"
-#endif
 
 /* insert the given .deb into the mirror in <component> in the <distribution>
  * putting things with architecture of "all" into <architectures> (and also
@@ -22,11 +19,12 @@
  * package. (forcesection and forcepriority have higher priority than the
  * information there),
  * if dereferencedfilekeys is != NULL, add there filekeys that lost a reference */
-retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,/*@null@*/const char *forcecomponent,/*@null@*/const char *forcearchitecture,/*@null@*/const char *forcesection,/*@null@*/const char *forcepriority,const char *packagetype,struct distribution *distribution,const char *debfilename,/*@null@*/const char *givenfilekey,/*@null@*/const char *givenmd5sum,/*@null@*/const struct overrideinfo *binoverride,int delete,/*@null@*/struct strlist *dereferencedfilekeys,/*@null@*/trackingdb);
+retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,/*@null@*/const char *forcecomponent,/*@null@*/const char *forcearchitecture,/*@null@*/const char *forcesection,/*@null@*/const char *forcepriority,const char *packagetype,struct distribution *distribution,const char *debfilename,/*@null@*/const char *givenfilekey,/*@null@*/const char *givenmd5sum,int delete,/*@null@*/struct strlist *dereferencedfilekeys,/*@null@*/trackingdb);
 
 /* in two steps */
 struct debpackage;
 retvalue deb_addprepared(const struct debpackage *pkg, const char *dbdir,references refs,const char *forcearchitecture,const char *packagetype,struct distribution *distribution,struct strlist *dereferencedfilekeys,struct trackingdata *trackingdata);
-retvalue deb_prepare(/*@out@*/struct debpackage **deb,filesdb filesdb,const char * const forcecomponent,const char * const forcearchitecture,const char *forcesection,const char *forcepriority,const char * const packagetype,struct distribution *distribution,const char *debfilename,const char * const givenfilekey,const char * const givenmd5sum,const struct overrideinfo *binoverride,int delete,bool_t needsourceversion,const struct strlist *allowed_binaries,const char *expectedsourcename,const char *expectedsourceversion);
+retvalue deb_hardlinkfiles(struct debpackage *deb,filesdb filesdb,const char *debfilename);
+retvalue deb_prepare(/*@out@*/struct debpackage **deb,filesdb filesdb,const char * const forcecomponent,const char * const forcearchitecture,const char *forcesection,const char *forcepriority,const char * const packagetype,struct distribution *distribution,const char *debfilename,const char * const givenfilekey,const char * const givenmd5sum,int delete,bool_t needsourceversion,const struct strlist *allowed_binaries,const char *expectedsourcename,const char *expectedsourceversion);
 void deb_free(/*@only@*/struct debpackage *pkg);
 #endif
