@@ -15,16 +15,24 @@
 #ifndef REPREPRO_APTMETHOD_H
 #include "aptmethod.h"
 #endif
+#ifndef REPREPRO_FREESPACE_H
+#include "freespace.h"
+#endif
 
-struct downloadcache;
+struct downloaditem;
+
+struct downloadcache {
+	/*@null@*/struct downloaditem *items;
+	/*@null@*/struct devices *devices;
+};
 
 /* Initialize a new download session */
-retvalue downloadcache_initialize(struct downloadcache **download);
+retvalue downloadcache_initialize(const char *dbdir,enum spacecheckmode mode,off_t reserveddb,off_t reservedother,struct downloadcache **download);
 
 /* free all memory */
 retvalue downloadcache_free(/*@null@*//*@only@*/struct downloadcache *download);
 
-/* queue a new file to be downloaded: 
+/* queue a new file to be downloaded:
  * results in RET_ERROR_WRONG_MD5, if someone else already asked
  * for the same destination with other md5sum created. */
 retvalue downloadcache_add(struct downloadcache *cache,filesdb filesdb,struct aptmethod *method,const char *orig,const char *filekey,const char *md5sum);
