@@ -18,9 +18,17 @@ typedef int bool_t;
 #define ISSET(a,b) (a&b)!=0
 #define NOTSET(a,b) (a&b)==0
 
+#ifdef STUPIDCC
+#define IFSTUPIDCC(a) a
+#else
+#define IFSTUPIDCC(a)
+#endif
+
 #ifdef SPLINT
 #define UNUSED(a) /*@unused@*/ a
+#define NORETURN
 #else
+#define NORETURN __attribute((noreturn))
 #ifndef NOUNUSEDATTRIBUTE
 #define UNUSED(a) a __attribute((unused))
 #else
@@ -28,8 +36,8 @@ typedef int bool_t;
 #endif
 #endif
 
-enum config_option_owner { 	CONFIG_OWNER_DEFAULT=0, 
-				CONFIG_OWNER_FILE, 
+enum config_option_owner { 	CONFIG_OWNER_DEFAULT=0,
+				CONFIG_OWNER_FILE,
 				CONFIG_OWNER_ENVIRONMENT,
 		           	CONFIG_OWNER_CMDLINE};
 #if LIBDB_VERSION == 44
