@@ -26,12 +26,13 @@
 	IGN(surprisingbinary) \
 	IGN(wrongsourceversion) \
 	IGN(wrongversion) \
+	IGN(dscinbinnmu) \
 	IGN(brokensignatures) \
 	IGN(uploaders) \
 	IGN(missingfile)
 
 
-enum ignore { 
+enum ignore {
 #define IGN(what) IGN_ ## what,
 	VALID_IGNORES
 #undef IGN
@@ -42,8 +43,8 @@ enum ignore {
 extern int ignored[IGN_COUNT];
 extern bool_t ignore[IGN_COUNT];
 
-#define IGNORING(ignoring,toignore,what, ...) \
-	({ 	fprintf(stderr, __VA_ARGS__); \
+#define IGNORING(ignoring,toignore,what,...) \
+	({ 	fprintf(stderr, ## __VA_ARGS__); \
 		ignored[IGN_ ## what] ++; \
 		if( ignore[IGN_ ## what] ) { \
 			fputs(ignoring " as --ignore=" #what " given.\n",stderr); \
@@ -52,7 +53,7 @@ extern bool_t ignore[IGN_COUNT];
 		} \
 		ignore[IGN_ ## what]; \
 	})
-#define IGNORING_(what, ...) IGNORING("Ignoring","To ignore",what, __VA_ARGS__ )
+#define IGNORING_(what,...) IGNORING("Ignoring","To ignore",what, __VA_ARGS__ )
 #define IGNORABLE(what) ignore[IGN_ ## what]
 
 #define RETURN_IF_ERROR_UNLESS_IGNORED(r,what,msg_fmt, ...) \
