@@ -1,7 +1,7 @@
 /*  This file is part of "reprepro"
  *  Copyright (C) 2003,2005 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as 
+ *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
@@ -132,7 +132,7 @@ retvalue md5sum_read(const char *filename,char **result){
 	}
 }
 
-retvalue md5sum_copy(const char *origfilename,const char *destfilename, 
+retvalue md5sum_copy(const char *origfilename,const char *destfilename,
 			char **result){
 	retvalue r;
 	int fdr,fdw;
@@ -183,7 +183,7 @@ retvalue md5sum_copy(const char *origfilename,const char *destfilename,
 	return RET_OK;
 }
 /* same as above, but delete existing files and try to hardlink first. */
-retvalue md5sum_place(const char *origfilename,const char *destfilename, 
+retvalue md5sum_place(const char *origfilename,const char *destfilename,
 			char **result) {
 	int i;
 	retvalue r;
@@ -200,7 +200,7 @@ retvalue md5sum_place(const char *origfilename,const char *destfilename,
 		if( verbose > 1 ) {
 			fprintf(stderr,"Linking failed, copying file '%s' to '%s'...\n",origfilename,destfilename);
 		}
-		
+
 		r = md5sum_copy(origfilename,destfilename,result);
 		if( r == RET_ERROR_EXIST ) {
 			fprintf(stderr,"File '%s' already exists and could not be removed to link/copy '%s' there!\n",destfilename,origfilename);
@@ -215,6 +215,7 @@ retvalue md5sum_ensure(const char *fullfilename,const char *md5sum,bool_t warnif
 	int fd,i;
 	struct stat s;
 	off_t expectedsize;
+	retvalue r;
 
 	assert(md5sum != NULL);
 
@@ -235,12 +236,11 @@ retvalue md5sum_ensure(const char *fullfilename,const char *md5sum,bool_t warnif
 		close(fd);
 		return RET_ERRNO(i);
 	}
-	// TODO: extract filesize from md5sum and compare here instead
-	// and change code below to check this number and not generate the 
-	// full string...
-	expectedsize = s.st_size;
+	r = calc_extractsize(md5sum, &expectedsize);
 
-	if( s.st_size == expectedsize ) {
+	/* if length cannot be parsed, just proceed, otherwise only
+	 * calculare the actual file checksum, if the length matched */
+	if( !RET_IS_OK(r) || s.st_size == expectedsize ) {
 		char *foundmd5sum;
 
 		ret = md5sum_calc(fd,-1,&foundmd5sum,0);
@@ -317,7 +317,7 @@ retvalue md5sum_replace(const char *filename, const char *data, size_t len, char
 
 	if( result != NULL ) {
 		MD5Init(&context);
-		MD5Update(&context,data,len);
+		MD5Update(&context,(const unsigned char*)data,len);
 		r = md5sum_genstring(&md5sum, &context,len);
 		assert( r != RET_NOTHING );
 		if( RET_WAS_ERROR(r) ) {
