@@ -1,7 +1,7 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2003,2004,2005,2006 Bernhard R. Link
+ *  Copyright (C) 2003,2004,2005,2006,2007 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as 
+ *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
@@ -21,6 +21,7 @@
 #include <ctype.h>
 #include <string.h>
 #include <malloc.h>
+#include <sys/types.h>
 #include "error.h"
 #include "ignore.h"
 #include "mprintf.h"
@@ -51,17 +52,17 @@ static inline bool_t overlongUTF8(const char *character) {
 
 		if( (c & (uchar)0x3E /* 00111110 */ ) == (uchar)0 )
 			return TRUE;
-		if( c == (uchar)0xE0 /*11100000*/ && 
-		    (nextc & (uchar)0x20 /*00100000*/ ) == (uchar)0) 
+		if( c == (uchar)0xE0 /*11100000*/ &&
+		    (nextc & (uchar)0x20 /*00100000*/ ) == (uchar)0)
 			return TRUE;
-		if( c == (uchar)0xF0 /*11110000*/ && 
-		    (nextc & (uchar)0x30 /*00110000*/ ) == (uchar)0) 
+		if( c == (uchar)0xF0 /*11110000*/ &&
+		    (nextc & (uchar)0x30 /*00110000*/ ) == (uchar)0)
 			return TRUE;
-		if( c == (uchar)0xF8 /*11111000*/ && 
-		    (nextc & (uchar)0x38 /*00111000*/ ) == (uchar)0) 
+		if( c == (uchar)0xF8 /*11111000*/ &&
+		    (nextc & (uchar)0x38 /*00111000*/ ) == (uchar)0)
 			return TRUE;
-		if( c == (uchar)0xFC /*11111100*/ && 
-		    (nextc & (uchar)0x3C /*00111100*/ ) == (uchar)0) 
+		if( c == (uchar)0xFC /*11111100*/ &&
+		    (nextc & (uchar)0x3C /*00111100*/ ) == (uchar)0)
 			return TRUE;
 	}
 	return FALSE;
@@ -82,7 +83,7 @@ static inline bool_t overlongUTF8(const char *character) {
 			*s,descr,string); \
 		return RET_ERROR; \
 	}
-	
+
 
 /* check if this is something that can be used as directory safely */
 retvalue propersourcename(const char *string) {
@@ -103,9 +104,9 @@ retvalue propersourcename(const char *string) {
 	}
 	s = string;
 	while( *s != '\0' ) {
-		if( (*s > 'z' || *s < 'a' ) && 
-		    (*s > '9' || *s < '0' ) && 
-		    (firstcharacter || 
+		if( (*s > 'z' || *s < 'a' ) &&
+		    (*s > '9' || *s < '0' ) &&
+		    (firstcharacter ||
 		     ( *s != '+' && *s != '-' && *s != '.'))) {
 			REJECTLOWCHARS(s,string,"sourcename");
 			REJECTCHARIF( *s == '/', s,string, "sourcename");
@@ -195,7 +196,7 @@ static retvalue properdirectoryandidentifier(const char *string, const char *des
 		return RET_ERROR;
 	}
 	if( (string[0] == '.' && (string[1] == '\0'||string[1]=='/')) ||
-		(string[0] == '.' && string[1] == '.' && 
+		(string[0] == '.' && string[1] == '.' &&
 		 	(string[2] == '\0'||string[2] =='/')) ) {
 		fprintf(stderr,"%s cannot be '%s', as it is used as directory!\n",description,string);
 		return RET_ERROR;
@@ -204,9 +205,9 @@ static retvalue properdirectoryandidentifier(const char *string, const char *des
 	while( *s != '\0' ) {
 		REJECTLOWCHARS(s,string,description);
 		REJECTCHARIF( *s == '|' ,s,string,description);
-		if( *s == '/' && 
+		if( *s == '/' &&
 			((s[1] == '.' && (s[2] == '\0' || s[2] == '/' ) ) ||
-			 (s[1] == '.' && s[2] == '.' && 
+			 (s[1] == '.' && s[2] == '.' &&
 			  	(s[3] == '\0' || s[3] =='/')))) {
 			fprintf(stderr,"%s cannot be '%s': directory parts . or .. in it!\n",description,string);
 			return RET_ERROR;
@@ -278,16 +279,16 @@ retvalue properversion(const char *string) {
 	}
 	if( ( *s < '0' || *s > '9' ) &&
 	    (( *s >= 'a' && *s <= 'z') || (*s >='A' && *s <= 'Z'))) {
-		/* As there are official packages violating the rule 
-		 * of policy 5.6.11 to start with a digit, disabling 
+		/* As there are official packages violating the rule
+		 * of policy 5.6.11 to start with a digit, disabling
 		 * this test, and only omitting a warning. */
-		if( verbose >= 0 ) 
+		if( verbose >= 0 )
 			fprintf(stderr,"Warning: Package version '%s' does not start with a digit, violating 'should'-directive in policy 5.6.11\n",string);
 	}
 	for( ; *s != '\0' ; s++,first=FALSE ) {
 		if( (*s <= '9' || *s >= '0' ) ) {
 			continue;
-		} 
+		}
 		if( !first && yetonlydigits && *s == ':' ) {
 			hadepoch = TRUE;
 			continue;
@@ -298,7 +299,7 @@ retvalue properversion(const char *string) {
 			yetonlydigits = FALSE;
 			continue;
 		}
-		if( first || (*s != '+'  && *s != '-' && 
+		if( first || (*s != '+'  && *s != '-' &&
 	            	      *s != '.'  && *s != '~' &&
 			      (!hadepoch || *s != ':' ))) {
 			REJECTLOWCHARS(s,string,"version");
@@ -317,7 +318,7 @@ retvalue properversion(const char *string) {
 					return RET_ERROR;
 				}
 			}
-		} 
+		}
 	}
 	return RET_OK;
 }
@@ -352,9 +353,9 @@ retvalue properpackagename(const char *string) {
 		/* DAK also allowed upper case letters last I looked, policy
 		 * does not, so they are not allowed without --ignore=forbiddenchar */
 		// perhaps some extra ignore-rule for upper case?
-		if( (*s > 'z' || *s < 'a' ) && 
-		    (*s > '9' || *s < '0' ) && 
-		    ( firstcharacter || 
+		if( (*s > 'z' || *s < 'a' ) &&
+		    (*s > '9' || *s < '0' ) &&
+		    ( firstcharacter ||
   	    	      (*s != '+' && *s != '-' && *s != '.'))) {
 			REJECTLOWCHARS(s,string,"package name");
 			REJECTCHARIF( *s == '/' ,s,string,"package name");
@@ -421,7 +422,7 @@ static inline char *escapecpy(char *dest,const char *orig) {
 char *calc_downloadedlistfile(const char *listdir,const char *codename,const char *origin,const char *component,const char *architecture,const char *packagetype) {
 	size_t l_listdir,len,l_packagetype;
 	char *result,*p;
-	
+
 	l_listdir = strlen(listdir),
 	len = escapedlen(codename) + escapedlen(origin) + escapedlen(component) + escapedlen(architecture);
 	l_packagetype = strlen(packagetype);
@@ -429,7 +430,7 @@ char *calc_downloadedlistfile(const char *listdir,const char *codename,const cha
 	if( result == NULL )
 		return result;
 	memcpy(p,listdir,l_listdir);
-	p += l_listdir; 
+	p += l_listdir;
 	*p = '/'; p++;
 	p = escapecpy(p,codename);
 	*p = '_'; p++;
@@ -447,7 +448,7 @@ char *calc_downloadedlistfile(const char *listdir,const char *codename,const cha
 char *calc_downloadedlistpattern(const char *codename) {
 	size_t len;
 	char *result,*p;
-	
+
 	len = escapedlen(codename);
 	p = result = malloc(len + 2);
 	if( result == NULL )
@@ -463,7 +464,7 @@ char *calc_identifier(const char *codename,const char *component,const char *arc
 	assert( strchr(codename,'|') == NULL && strchr(component,'|') == NULL && strchr(architecture,'|') == NULL );
 	assert( codename != NULL ); assert( component != NULL );
 	assert( architecture != NULL ); assert( packagetype != NULL );
-	if( packagetype[0] == 'u' ) 
+	if( packagetype[0] == 'u' )
 		return mprintf("u|%s|%s|%s",codename,component,architecture);
 	else
 		return mprintf("%s|%s|%s",codename,component,architecture);
@@ -479,7 +480,7 @@ char *calc_dirconcat(const char *str1,const char *str2) {
 char *calc_dirconcatn(const char *str1,const char *str2,size_t len2) {
 	size_t len1;
 	char *r;
-	
+
 	assert(str1 != NULL);
 	assert(str2 != NULL);
 	len1 = strlen(str1);
@@ -502,10 +503,6 @@ char *calc_dirsuffixconcat(const char *str1,const char *str2,const char *suffix)
 	return mprintf("%s/%s.%s",str1,str2,suffix);
 }
 
-char *calc_srcfilekey(const char *sourcedir,const char *filename){
-	return calc_dirconcat(sourcedir,filename);
-}
-
 char *calc_fullfilename(const char *mirrordir,const char *filekey){
 	return calc_dirconcat(mirrordir,filekey);
 }
@@ -558,6 +555,38 @@ char *calc_source_basename(const char *name,const char *version) {
 	return mprintf("%s_%s.dsc",name,v);
 }
 
+retvalue calc_extractsize(const char *checksum, off_t *size) {
+	off_t value;
+#if 0
+	/* over possible extensions (not yet implemented otherwise) */
+	while( *checksum == ':' ) {
+		checksum++;
+		while( *checksum != ' ' && *checksum != '\0' )
+			checksum++;
+		if( *checksum == ' ' )
+			checksum++;
+	}
+#endif
+	/* over md5 part */
+	while( *checksum != ' ' && *checksum != '\0' )
+		checksum++;
+	if( *checksum == ' ' )
+		checksum++;
+	if( *checksum < '0' || *checksum > '9' )
+		return RET_NOTHING;
+	value = *checksum - '0';
+	checksum++;
+	while( *checksum >= '0' && *checksum <= '9' ) {
+		value *= 10;
+		value += (*checksum - '0');
+		checksum++;
+	}
+	if( *checksum != '\0' )
+		return RET_NOTHING;
+	*size = value;
+	return RET_OK;
+}
+
 char *calc_concatmd5andsize(const char *md5sum,const char *size) {
 	/* this is not the only reference, as there are prints
 	 * with size as ofs_t, too */
@@ -574,7 +603,7 @@ char *names_concatmd5sumandsize(const char *md5start,const char *md5end,const ch
 	result[(md5end-md5start)] = ' ';
 	memcpy(result+(md5end-md5start)+1,sizestart,sizeend-sizestart);
 	result[(md5end-md5start)+1+(sizeend-sizestart)] = '\0';
-	
+
 	return result;
 
 }
@@ -617,14 +646,6 @@ void names_overversion(const char **version,bool_t epochsuppressed) {
 	if( *n < '0' || *n > '9' ) {
 		if( (*n < 'a' || *n > 'z') && (*n < 'A' || *n > 'Z') )
 			return;
-		else {
-			/* As there are packages violating the rule of policy 5.6.11 to
-			 * start with a digit, disabling this test, and only omitting a
-			 * warning. */
-			if( verbose >= 0 ) 
-				fprintf(stderr,"Warning: Package version '%s' does not start with a digit, violating 'should'-directive in policy 5.6.11\n",n);
-//			return;
-		}
 	} else
 		n++;
 	while( *n >= '0' && *n <= '9' )
@@ -632,7 +653,6 @@ void names_overversion(const char **version,bool_t epochsuppressed) {
 	if( *n == ':' ) {
 		hadepoch = TRUE;
 		n++;
-//TODO: more corectly another check should be here to also look for a digit...
 	}
 	while( ( *n >= '0' && *n <= '9' ) || ( *n >= 'a' && *n <= 'z')
 			|| ( *n >= 'A' && *n <= 'Z' ) || *n == '.' || *n == '~'
@@ -697,7 +717,7 @@ retvalue calc_parsefileline(const char *fileline,char **filename,char **md5sum)
 		md5as[md5end-md5] = ' ';
 		strncpy(md5as+1+(md5end-md5),size,sizeend-size);
 		md5as[(md5end-md5)+1+(sizeend-size)] = '\0';
-	
+
 		*md5sum = md5as;
 	}
 	if( filename != NULL )
@@ -718,36 +738,29 @@ retvalue splitlist(struct strlist *from,
 	retvalue r;
 	int i;
 
-	r = strlist_init(from);
-	if( RET_WAS_ERROR(r) ) {
-		return r;
-	}
-	r = strlist_init(into);
-	if( RET_WAS_ERROR(r) ) {
-		strlist_done(from);
-		return r;
-	}
+	strlist_init(from);
+	strlist_init(into);
 
 	/* * Iterator over components to update * */
 	r = RET_NOTHING;
 	for( i = 0 ; i < list->count ; i++ ) {
-		const char *item,*seperator;
+		const char *item,*separator;
 		char *origin,*destination;
 
 		item = list->values[i];
 		// TODO: isn't this broken for the !*(dest+1) case ?
-		if( (seperator = strchr(item,'>')) == NULL ) {
+		if( (separator = strchr(item,'>')) == NULL ) {
 			destination = strdup(item);
 			origin = strdup(item);
-		} else if( seperator == item ) {
-			destination = strdup(seperator+1);
-			origin = strdup(seperator+1);
-		} else if( *(seperator+1) == '\0' ) {
-			destination = strndup(item,seperator-item);
-			origin = strndup(item,seperator-item);
+		} else if( separator == item ) {
+			destination = strdup(separator+1);
+			origin = strdup(separator+1);
+		} else if( *(separator+1) == '\0' ) {
+			destination = strndup(item,separator-item);
+			origin = strndup(item,separator-item);
 		} else {
-			origin = strndup(item,seperator-item);
-			destination = strdup(seperator+1);
+			origin = strndup(item,separator-item);
+			destination = strdup(separator+1);
 		}
 		if( origin == NULL || destination == NULL ) {
 			free(origin);free(destination);
@@ -773,3 +786,39 @@ retvalue splitlist(struct strlist *from,
 	return r;
 }
 
+char *calc_changes_basename(const char *name,const char *version,const struct strlist *architectures) {
+	size_t name_l, version_l, l;
+	int i;
+	char *n,*p;
+
+	name_l = strlen(name);
+	version_l = strlen(version);
+	l = name_l + version_l + sizeof("__.changes");
+
+	for( i = 0 ; i < architectures->count ; i++ ) {
+		l += strlen(architectures->values[i]);
+		if( i != 0 )
+			l++;
+	}
+	n = malloc(l);
+	if( n == NULL )
+		return n;
+	p = n;
+	memcpy(p, name, name_l); p+=name_l;
+	*(p++) = '_';
+	memcpy(p, version, version_l); p+=version_l;
+	*(p++) = '_';
+	for( i = 0 ; i < architectures->count ; i++ ) {
+		size_t a_l = strlen(architectures->values[i]);
+		if( i != 0 )
+			*(p++) = '+';
+		assert( (size_t)((p+a_l)-n) < l );
+		memcpy(p, architectures->values[i], a_l);
+		p += a_l;
+	}
+	assert( (size_t)(p-n) < l-8 );
+	memcpy(p, ".changes", 9 ); p += 9;
+	assert( *(p-1) == '\0' );
+	assert( (size_t)(p-n) == l );
+	return n;
+}
