@@ -14,14 +14,15 @@ char *calc_dirconcat3(const char *str1,const char *str2,const char *str3);
 
 char *calc_binary_basename(const char *package,const char *version,const char *architecture,const char *packagetype);
 char *calc_source_basename(const char *name,const char *version);
+char *calc_changes_basename(const char *name,const char *version,const struct strlist *architectures);
 char *calc_sourcedir(const char *component,const char *sourcename);
 char *calc_filekey(const char *component,const char *sourcename,const char *filename);
-char *calc_srcfilekey(const char *sourcedir,const char *filename);
 char *calc_fullfilename(const char *mirrordir,const char *filekey);
 char *calc_fullsrcfilename(const char *mirrordir,const char *directory,const char *filename);
 char *calc_identifier(const char *codename,const char *component,const char *architecture,const char *packagetype);
 char *calc_concatmd5andsize(const char *md5sum,const char *size);
 char *names_concatmd5sumandsize(const char *md5start,const char *md5end,const char *sizestart,const char *sizeend);
+retvalue calc_extractsize(const char *checksum, /*@out@*/off_t *size);
 char *calc_trackreferee(const char *codename,const char *sourcename,const char *sourceversion);
 
 char *calc_downloadedlistfile(const char *listdir,const char *codename,const char *origin,const char *component,const char *architecture,const char *packagetype);
