@@ -1,9 +1,8 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2003,2004,2005 Bernhard R. Link
+ *  Copyright (C) 2003,2004,2005,2007 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,7 +50,7 @@ retvalue references_done(references refs) {
 	else
 		return RET_DBERR(r);
 }
-	
+
 retvalue references_initialize(references *refs,const char *dbpath) {
 	int ret;
 	char *filename;
@@ -62,7 +61,7 @@ retvalue references_initialize(references *refs,const char *dbpath) {
 	ref = malloc(sizeof(*ref));
 	if( ref == NULL )
 		return RET_ERROR_OOM;
-	
+
 	filename = calc_dirconcat(dbpath,"references.db");
 	if( filename == NULL ) {
 		free(ref);
@@ -88,7 +87,8 @@ retvalue references_initialize(references *refs,const char *dbpath) {
 		free(ref);
 		return RET_DBERR(ret);
 	}
-	if( (ret = ref->db->open(ref->db, filename, "references", DB_BTREE, DB_CREATE, 0664)) != 0) {
+	ret = DB_OPEN(ref->db, filename, "references", DB_BTREE, DB_CREATE);
+	if( ret != 0) {
 		ref->db->err(ref->db, ret, "db_open:%s", filename);
 		(void)ref->db->close(ref->db,0);
 		free(filename);
@@ -105,7 +105,7 @@ retvalue references_isused(references refs,const char *what) {
 	DBT key,data;
 
 	SETDBT(key,what);
-	CLEARDBT(data);	
+	CLEARDBT(data);
 	if( (dbret = refs->db->get(refs->db, NULL, &key, &data, 0)) == 0){
 		return RET_OK;
 	} else if( dbret == DB_NOTFOUND ){
@@ -127,8 +127,8 @@ static retvalue references_checksingle(references refs,const char *what,const ch
 		refs->db->err(refs->db, dbret, "references_check dberror:");
 		return RET_DBERR(dbret);
 	}
-	SETDBT(key,what);	
-	SETDBT(data,by);	
+	SETDBT(key,what);
+	SETDBT(data,by);
 	if( (dbret=cursor->c_get(cursor,&key,&data,DB_GET_BOTH)) == 0 ) {
 		r = RET_OK;
 	} else
@@ -153,7 +153,7 @@ retvalue references_check(references refs,const char *referee,const struct strli
 	for( i = 0 ; i < filekeys->count ; i++ ) {
 		r = references_checksingle(refs,filekeys->values[i],referee);
 		RET_UPDATE(ret,r);
-		
+
 	}
 	return ret;
 }
@@ -167,7 +167,7 @@ retvalue references_increment(references refs,const char *needed,const char *nee
 	SETDBT(data,neededby);
 	if ((dbret = refs->db->put(refs->db, NULL, &key, &data, 0)) == 0) {
 		if( verbose > 8 )
-			fprintf(stderr,"Adding reference to '%s' by '%s'\n", needed,neededby);
+			printf("Adding reference to '%s' by '%s'\n", needed,neededby);
 		return RET_OK;
 	} else {
 		refs->db->err(refs->db, dbret, "references_increment dberror:");
@@ -188,8 +188,8 @@ retvalue references_decrement(references refs,const char *needed,const char *nee
 		refs->db->err(refs->db, dbret, "references_decrement dberror:");
 		return RET_DBERR(dbret);
 	}
-	SETDBT(key,needed);	
-	SETDBT(data,neededby);	
+	SETDBT(key,needed);
+	SETDBT(data,neededby);
 	if( (dbret=cursor->c_get(cursor,&key,&data,DB_GET_BOTH)) == 0 ) {
 			if( verbose > 8 )
 				fprintf(stderr,"Removing reference to '%s' by '%s'\n",
@@ -199,7 +199,7 @@ retvalue references_decrement(references refs,const char *needed,const char *nee
 				refs->db->err(refs->db, dbret, "references_decrement dberror(del):");
 				RET_UPDATE(r,RET_DBERR(dbret));
 			}
-	} else 
+	} else
 	if( dbret != DB_NOTFOUND ) {
 		refs->db->err(refs->db, dbret, "references_decrement dberror(get):");
 		return RET_DBERR(dbret);
@@ -231,6 +231,27 @@ retvalue references_insert(references refs,const char *identifier,
 	return result;
 }
 
+/* add possible already existing references */
+retvalue references_add(references refs,const char *identifier,const struct strlist *files) {
+	int dbret;
+	int i;
+	DBT key,data;
+
+	for( i = 0 ; i < files->count ; i++ ) {
+		const char *filekey = files->values[i];
+		SETDBT(key,filekey);
+		SETDBT(data,identifier);
+		if ((dbret = refs->db->put(refs->db, NULL, &key, &data, DB_NODUPDATA)) == 0) {
+			if( verbose > 8 )
+				fprintf(stderr,"Added reference to '%s' by '%s'\n", filekey, identifier);
+		} else if( dbret != DB_KEYEXIST) {
+			refs->db->err(refs->db, dbret, "references_add dberror:");
+			return RET_DBERR(dbret);
+		}
+	}
+	return RET_OK;
+}
+
 /* Remove reference by <identifer> for the given <oldfiles>,
  * excluding <exclude>, if it is nonNULL. */
 retvalue references_delete(references refs,const char *identifier,
@@ -260,11 +281,12 @@ retvalue references_delete(references refs,const char *identifier,
 	}
 	strlist_done(files);
 	return result;
-	
+
 }
 
 /* remove all references from a given identifier */
-retvalue references_remove(references refs,const char *neededby) {
+retvalue references_remove(references refs,const char *neededby,
+		struct strlist *dereferenced) {
 	DBC *cursor;
 	DBT key,data;
 	int dbret;
@@ -278,18 +300,31 @@ retvalue references_remove(references refs,const char *neededby) {
 		return RET_DBERR(dbret);
 	}
 	l = strlen(neededby);
-	CLEARDBT(key);	
-	CLEARDBT(data);	
+	CLEARDBT(key);
+	CLEARDBT(data);
 	while( (dbret=cursor->c_get(cursor,&key,&data,DB_NEXT)) == 0 ) {
 		const char *found_to = key.data;
 		const char *found_by = data.data;
-		if( strncmp( found_by,neededby,l) == 0 && 
+		if( strncmp( found_by,neededby,l) == 0 &&
 		    (found_by[l] == '\0' || found_by[l] == ' ')) {
 			if( verbose > 8 )
 				fprintf(stderr,"Removing reference to '%s' by '%s'\n",
 					found_to,neededby);
 			dbret = cursor->c_del(cursor,0);
-			if( dbret != 0 ) {
+			if( dbret == 0 ) {
+				if( dereferenced != NULL ) {
+					char *f = strndup(key.data,key.size-1);
+					retvalue r2;
+					if( f == NULL ) {
+						r = RET_ERROR_OOM;
+						continue;
+					}
+					r2 = strlist_add(dereferenced, f);
+					if( RET_WAS_ERROR(r2) ) {
+						r = r2;
+					}
+				}
+			} else {
 				refs->db->err(refs->db, dbret, "references_remove dberror(del):");
 				RET_UPDATE(r,RET_DBERR(dbret));
 			}
@@ -318,13 +353,13 @@ retvalue references_dump(references refs) {
 		refs->db->err(refs->db, dbret, "references_dump dberror(cursor):");
 		return RET_DBERR(dbret);
 	}
-	CLEARDBT(key);	
+	CLEARDBT(key);
 	CLEARDBT(data);
 	result = RET_NOTHING;
 	while( (dbret=cursor->c_get(cursor,&key,&data,DB_NEXT)) == 0 ) {
 		const char *found_to = (const char*)key.data;
 		const char *found_by = (const char*)data.data;
-		if( fputs(found_by,stdout) == EOF || 
+		if( fputs(found_by,stdout) == EOF ||
 		    putchar(' ') == EOF ||
 		    puts(found_to) == EOF ) {
 			result = RET_ERROR;
