@@ -18,13 +18,17 @@ retvalue references_initialize(/*@out@*/references *ref, const char *path);
 retvalue references_done(/*@only@*/references ref);
 
 /* remove all references from a given identifier */
-retvalue references_remove(references ref,const char *needey);
+retvalue references_remove(references ref,const char *neededby,/*@null@*/struct strlist *);
 
 /* Add an reference by <identifer> for the given <files>,
  * excluding <exclude>, if it is nonNULL. */
 retvalue references_insert(references ref,const char *identifer,
 		const struct strlist *files,const struct strlist *exclude);
 
+/* Add an reference by <identifer> for the given <files>,
+ * do not error out if reference already exists */
+retvalue references_add(references ref,const char *identifer,const struct strlist *files);
+
 /* Remove reference by <identifer> for the given <oldfiles>,
  * excluding <exclude>, if it is nonNULL.
  * if dereferencedfilekeys is != NULL, add those losing one reference,
