@@ -1,7 +1,7 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2003,2004,2005 Bernhard R. Link
+ *  Copyright (C) 2003,2004,2005,2007 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as 
+ *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
@@ -54,6 +54,8 @@ struct release {
 	char *dirofdist;
 	/* anything new yet added */
 	bool_t new;
+	/* snapshot */
+	bool_t snapshot;
 	/* the files yet for the list */
 	struct release_entry {
 		struct release_entry *next;
@@ -65,10 +67,6 @@ struct release {
 	} *files;
 	/* the cache database for old files */
 	DB *cachedb;
-	/* some more things, only here to make
-	 * free'ing them in case of error easier */
-	char *newreleasefilename,*releasefilename;
-	char *newsignfilename,*signfilename;
 };
 
 void release_free(struct release *release) {
@@ -83,10 +81,6 @@ void release_free(struct release *release) {
 		free(e->fulltemporaryfilename);
 		free(e);
 	}
-	free(release->newreleasefilename);
-	free(release->releasefilename);
-	free(release->newsignfilename);
-	free(release->signfilename);
 	if( release->cachedb != NULL ) {
 		release->cachedb->close(release->cachedb,0);
 	}
@@ -97,7 +91,7 @@ const char *release_dirofdist(struct release *release) {
 	return release->dirofdist;
 }
 
-static retvalue newreleaseentry(struct release *release, /*@only@*/ char *relativefilename, 
+static retvalue newreleaseentry(struct release *release, /*@only@*/ char *relativefilename,
 		/*@only@*/ char *md5sum,
 		/*@only@*/ /*@null@*/ char *fullfinalfilename,
 		/*@only@*/ /*@null@*/ char *fulltemporaryfilename ) {
@@ -160,6 +154,21 @@ retvalue release_init(const char *dbdir, const char *distdir, const char *codena
 	return RET_OK;
 }
 
+retvalue release_initsnapshot(const char *distdir, const char *codename, const char *name, struct release **release) {
+	struct release *n;
+
+	n = calloc(1,sizeof(struct release));
+	n->dirofdist = mprintf("%s/%s/snapshots/%s",distdir,codename,name);
+	if( n->dirofdist == NULL ) {
+		free(n);
+		return RET_ERROR_OOM;
+	}
+	n->cachedb = NULL;
+	n->snapshot = TRUE;
+	*release = n;
+	return RET_OK;
+}
+
 retvalue release_adddel(struct release *release,/*@only@*/char *reltmpfile) {
 	char *filename;
 
@@ -258,7 +267,7 @@ static retvalue release_usecached(struct release *release,
 #ifdef HAVE_LIBBZ2
 	if( (compressions & IC_FLAG(ic_bzip2)) != 0 ) {
 		char *bzfilename;
-		bzfilename = calc_addsuffix(filename,"bz");
+		bzfilename = calc_addsuffix(filename,"bz2");
 		if( bzfilename == NULL ) {
 			free(filename);
 			return RET_ERROR_OOM;
@@ -358,7 +367,7 @@ struct filetorelease {
 #endif
 #ifdef HAVE_LIBBZ2
 	/* output buffer for bzip2 compression */
-	unsigned char *bzoutputbuffer; size_t bz_waiting_bytes;
+	char *bzoutputbuffer; size_t bz_waiting_bytes;
 	bz_stream bzstream;
 #endif
 };
@@ -421,7 +430,7 @@ static retvalue openfile(const char *dirofdist, struct openfile *f) {
 	return RET_OK;
 }
 
-static retvalue writetofile(struct openfile *file, const char *data, size_t len) {
+static retvalue writetofile(struct openfile *file, const unsigned char *data, size_t len) {
 
 	file->filesize += len;
 	MD5Update(&file->context,data,len);
@@ -458,13 +467,13 @@ static retvalue	initgzcompression(struct filetorelease *f) {
 	}
 #else
 	unsigned char header[10] = {
-		31,139, Z_DEFLATED, 
+		31,139, Z_DEFLATED,
 		/* flags */
-		0, 
+		0,
 		/* time */
 		0, 0, 0, 0,
 		/* xtra-flags */
-		0, 	
+		0,
 		/* os (3 = unix, 255 = unknown)
 		 * using unknown to generate the same file
 		 * with or without OLDZLIB */
@@ -487,7 +496,7 @@ static retvalue	initgzcompression(struct filetorelease *f) {
 	f->gzstream.opaque = NULL;
 #ifndef OLDZLIB
 	zret = deflateInit2(&f->gzstream,
-			/* Level: 0-9 or Z_DEFAULT_COMPRESSION: */ 
+			/* Level: 0-9 or Z_DEFAULT_COMPRESSION: */
 			Z_DEFAULT_COMPRESSION,
 			/* only possibility yet: */
 			Z_DEFLATED,
@@ -500,7 +509,7 @@ static retvalue	initgzcompression(struct filetorelease *f) {
 			);
 #else
 	zret = deflateInit2(&f->gzstream,
-			/* Level: 0-9 or Z_DEFAULT_COMPRESSION: */ 
+			/* Level: 0-9 or Z_DEFAULT_COMPRESSION: */
 			Z_DEFAULT_COMPRESSION,
 			/* only possibility yet: */
 			Z_DEFLATED,
@@ -563,8 +572,8 @@ static retvalue	initbzcompression(struct filetorelease *f) {
 }
 #endif
 
-static retvalue startfile(struct release *release, 
-		char *filename, compressionset compressions, 
+static retvalue startfile(struct release *release,
+		char *filename, compressionset compressions,
 		bool_t usecache,
 		struct filetorelease **file) {
 	struct filetorelease *n;
@@ -646,8 +655,8 @@ static retvalue startfile(struct release *release,
 	return RET_OK;
 }
 
-retvalue release_startfile2(struct release *release, 
-		const char *relative_dir, const char *filename, compressionset compressions, 
+retvalue release_startfile2(struct release *release,
+		const char *relative_dir, const char *filename, compressionset compressions,
 		bool_t usecache,
 		struct filetorelease **file) {
 	char *relfilename;
@@ -657,8 +666,8 @@ retvalue release_startfile2(struct release *release,
 		return RET_ERROR_OOM;
 	return startfile(release,relfilename,compressions,usecache,file);
 }
-retvalue release_startfile(struct release *release, 
-		const char *filename, compressionset compressions, 
+retvalue release_startfile(struct release *release,
+		const char *filename, compressionset compressions,
 		bool_t usecache,
 		struct filetorelease **file) {
 	char *relfilename;
@@ -678,9 +687,9 @@ static retvalue releasefile(struct release *release, struct openfile *f) {
 		assert( f->fulltemporaryfilename == NULL);
 		return RET_NOTHING;
 	}
-	assert((f->fullfinalfilename == NULL 
+	assert((f->fullfinalfilename == NULL
 		  && f->fulltemporaryfilename == NULL)
-	  	|| (f->fullfinalfilename != NULL 
+	  	|| (f->fullfinalfilename != NULL
 		  && f->fulltemporaryfilename != NULL));
 
 	r = md5sum_genstring(&md5sum,&f->context,f->filesize);
@@ -688,7 +697,7 @@ static retvalue releasefile(struct release *release, struct openfile *f) {
 	if( RET_WAS_ERROR(r) )
 		return r;
 	r = newreleaseentry(release,f->relativefilename,md5sum,
-			f->fullfinalfilename, 
+			f->fullfinalfilename,
 			f->fulltemporaryfilename);
 	f->relativefilename = NULL;
 	f->fullfinalfilename = NULL;
@@ -707,8 +716,8 @@ static retvalue writegz(struct filetorelease *f, const char *data, size_t len) {
 #ifdef OLDZLIB
 	f->crc = crc32(f->crc,data,len);
 #endif
-	
-	f->gzstream.next_in = (char*)data;
+
+	f->gzstream.next_in = (unsigned char*)data;
 	f->gzstream.avail_in = len;
 
 	do {
@@ -736,7 +745,7 @@ static retvalue writegz(struct filetorelease *f, const char *data, size_t len) {
 		 * should only happend when no output is possible, as that
 		 * gets possible again it should finally produce more output
 		 * and return Z_OK and always terminate. Hopefully... */
-	} while( zret == Z_BUF_ERROR 
+	} while( zret == Z_BUF_ERROR
 			|| ( zret == Z_OK && f->gzstream.avail_in != 0));
 
 	f->gzstream.next_in = NULL;
@@ -757,7 +766,7 @@ static retvalue writegz(struct filetorelease *f, const char *data, size_t len) {
 
 static retvalue finishgz(struct filetorelease *f) {
 	int zret;
-	char dummy;
+	unsigned char dummy;
 #ifdef OLDZLIB
 	unsigned char buffer[4];
 	retvalue r;
@@ -861,12 +870,13 @@ static retvalue writebz(struct filetorelease *f, const char *data, size_t len) {
 		bzret = BZ2_bzCompress(&f->bzstream,BZ_RUN);
 		f->bz_waiting_bytes = BZBUFSIZE - f->bzstream.avail_out;
 
-		if( bzret == BZ_RUN_OK && 
+		if( bzret == BZ_RUN_OK &&
 				f->bz_waiting_bytes >= BZBUFSIZE / 2 ) {
 			retvalue r;
 			assert( f->bz_waiting_bytes > 0 );
 			r = writetofile(&f->f[ic_bzip2],
-					f->bzoutputbuffer, f->bz_waiting_bytes);
+					(const unsigned char *)f->bzoutputbuffer,
+					f->bz_waiting_bytes);
 			assert( r != RET_NOTHING );
 			if( RET_WAS_ERROR(r) )
 				return r;
@@ -905,7 +915,8 @@ static retvalue finishbz(struct filetorelease *f) {
 			retvalue r;
 			assert( f->bz_waiting_bytes != 0 );
 			r = writetofile(&f->f[ic_bzip2],
-					f->bzoutputbuffer, f->bz_waiting_bytes);
+					(const unsigned char*)f->bzoutputbuffer,
+					f->bz_waiting_bytes);
 			assert( r != RET_NOTHING );
 			if( RET_WAS_ERROR(r) )
 				return r;
@@ -1003,10 +1014,10 @@ retvalue release_finishfile(struct release *release, struct filetorelease *file)
 
 retvalue release_writedata(struct filetorelease *file, const char *data, size_t len) {
 	retvalue result,r;
- 
+
 	result = RET_OK;
 	/* always call this so that md5sum is calculated */
-	r = writetofile(&file->f[ic_uncompressed],data,len);
+	r = writetofile(&file->f[ic_uncompressed],(const unsigned char*)data,len);
 	RET_UPDATE(result,r);
 	if( file->f[ic_gzip].relativefilename != NULL ) {
 		r = writegz(file,data,len);
@@ -1050,6 +1061,7 @@ retvalue release_directorydescription(struct release *release, const struct dist
 	release_writeheader("Architecture",target->architecture);
 	release_writeheader("NotAutomatic",distribution->notautomatic);
 	release_writeheader("Description",distribution->description);
+#undef release_writeheader
 	r = release_finishfile(release, f);
 	return r;
 }
@@ -1061,6 +1073,9 @@ static retvalue getcachevalue(struct release *r,const char *relfilename, char **
 	int dbret;
 	DBT key,data;
 
+	if( r->cachedb == NULL )
+		return RET_NOTHING;
+
 	SETDBT(key,relfilename);
 	CLEARDBT(data);
 	dbret = r->cachedb->get(r->cachedb, NULL, &key, &data, 0);
@@ -1113,10 +1128,8 @@ static retvalue storechecksums(struct release *r) {
 #undef CLEARDBT
 #undef SETDBT
 
-
 /* Generate a main "Release" file for a distribution */
 retvalue release_write(/*@only@*/struct release *release, struct distribution *distribution, bool_t onlyifneeded) {
-	FILE *f;
 	size_t s;
 	int e;
 	retvalue result,r;
@@ -1126,6 +1139,7 @@ retvalue release_write(/*@only@*/struct release *release, struct distribution *d
 	int i;
 	struct release_entry *file;
 	bool_t somethingwasdone;
+	struct signedfile *signedfile;
 
 	if( onlyifneeded && !release->new ) {
 		release_free(release);
@@ -1145,161 +1159,81 @@ retvalue release_write(/*@only@*/struct release *release, struct distribution *d
 		return RET_ERROR;
 	}
 
-	release->newreleasefilename = calc_dirconcat(release->dirofdist,"Release.new");
-	if( release->newreleasefilename == NULL ) {
+	if( distribution->signwith != NULL )
+		r = signature_startsignedfile(release->dirofdist, "Release",
+				distribution->signwith,
+				&signedfile);
+	else
+		r = signature_startunsignedfile(release->dirofdist, "Release",
+				&signedfile);
+	if( RET_WAS_ERROR(r) ) {
 		release_free(release);
-		return RET_ERROR_OOM;
-	}
-	release->releasefilename = calc_dirconcat(release->dirofdist,"Release");
-	if( release->releasefilename == NULL ) {
-		release_free(release);
-		return RET_ERROR_OOM;
-	}
-	(void)dirs_make_parent(release->newreleasefilename);
-	(void)unlink(release->newreleasefilename);
-	f = fopen(release->newreleasefilename,"w");
-	if( f == NULL ) {
-		e = errno;
-		fprintf(stderr,"Error writing file %s: %m\n",release->newreleasefilename);
-		release_free(release);
-		return RET_ERRNO(e);
-	}
-#define checkwritten if( e < 0 ) { \
-		e = errno; \
-		fprintf(stderr,"Error writing to %s: %d=$m!\n",release->newreleasefilename,e); \
-		release_free(release); \
-		(void)fclose(f); \
-		return RET_ERRNO(e); \
+		return r;
 	}
+#define writestring(s) signedfile_write(signedfile, s, strlen(s))
+#define writechar(c) {char __c = c ; signedfile_write(signedfile, &__c, 1); }
 
 	if( distribution->origin != NULL ) {
-		e = fputs("Origin: ",f);
-		checkwritten;
-		e = fputs(distribution->origin,f);
-		checkwritten;
-		e = fputc('\n',f) - 1;
-		checkwritten;
+		writestring("Origin: ");
+		writestring(distribution->origin);
+		writechar('\n');
 	}
 	if( distribution->label != NULL ) {
-		e = fputs("Label: ",f);
-		checkwritten;
-		e = fputs(distribution->label,f);
-		checkwritten;
-		e = fputc('\n',f) - 1;
-		checkwritten;
+		writestring("Label: ");
+		writestring(distribution->label);
+		writechar('\n');
 	}
 	if( distribution->suite != NULL ) {
-		e = fputs("Suite: ",f);
-		checkwritten;
-		e = fputs(distribution->suite,f);
-		checkwritten;
-		e = fputc('\n',f) - 1;
-		checkwritten;
+		writestring("Suite: ");
+		writestring(distribution->suite);
+		writechar('\n');
 	}
-	e = fputs("Codename: ",f);
-	checkwritten;
-	e = fputs(distribution->codename,f);
-	checkwritten;
+	writestring("Codename: ");
+	writestring(distribution->codename);
 	if( distribution->version != NULL ) {
-		e = fputs("\nVersion: ",f);
-		checkwritten;
-		e = fputs(distribution->version,f);
-		checkwritten;
+		writestring("\nVersion: ");
+		writestring(distribution->version);
 	}
-	e = fputs("\nDate: ",f);
-	checkwritten;
-	e = fputs(buffer,f);
-	checkwritten;
-	e = fputs("\nArchitectures:",f);
-	checkwritten;
+	writestring("\nDate: ");
+	writestring(buffer);
+	writestring("\nArchitectures:");
 	for( i = 0 ; i < distribution->architectures.count ; i++ ) {
 		/* Debian's topmost Release files do not list it, so we won't either */
 		if( strcmp(distribution->architectures.values[i],"source") == 0 )
 			continue;
-		e = fputc(' ',f) - 1;
-		checkwritten;
-		e = fputs(distribution->architectures.values[i],f);
-		checkwritten;
+		writechar(' ');
+		writestring(distribution->architectures.values[i]);
 	}
-	e = fputs("\nComponents: ",f);
-	checkwritten;
-	r = strlist_fprint(f,&distribution->components);
-	if( RET_WAS_ERROR(r) ) {
-		fprintf(stderr,"Error writing to %s!\n",release->newreleasefilename);
-		release_free(release);
-		(void)fclose(f);
-		return r;
+	writestring("\nComponents:");
+	for( i = 0 ; i < distribution->components.count ; i++ ) {
+		writechar(' ');
+		writestring(distribution->components.values[i]);
 	}
 	if( distribution->description != NULL ) {
-		e = fputs("\nDescription: ",f);
-		checkwritten;
-		e = fputs(distribution->description,f);
-		checkwritten;
+		writestring("\nDescription: ");
+		writestring(distribution->description);
 	}
 	if( distribution->notautomatic != NULL ) {
-		e = fputs("\nNotAutomatic: ",f);
-		checkwritten;
-		e = fputs(distribution->notautomatic,f);
-		checkwritten;
+		writestring("\nNotAutomatic: ");
+		writestring(distribution->notautomatic);
 	}
 
-	e = fputs("\nMD5Sum:\n",f);
-	checkwritten;
-
+	writestring("\nMD5Sum:\n");
 	for( file = release->files ; file != NULL ; file = file->next ) {
 		if( file->md5sum != NULL && file->relativefilename != NULL ) {
-			e = fputc(' ',f) - 1;
-			checkwritten;
-			e = fputs(file->md5sum,f);
-			checkwritten;
-			e = fputc(' ',f) - 1;
-			checkwritten;
-			e = fputs(file->relativefilename,f);
-			checkwritten;
-			e = fputc('\n',f) - 1;
-			checkwritten;
+			writechar(' ');
+			writestring(file->md5sum);
+			writechar(' ');
+			writestring(file->relativefilename);
+			writechar('\n');
 		}
 	}
-
-#undef checkwritten
-	if( ferror(f) != 0 ) { 
-		e = errno; 
-		fprintf(stderr,"Error writing to %s: %d=$m!\n",
-				release->newreleasefilename,e);
+	r = signedfile_prepare(signedfile, distribution->signwith);
+	if( RET_WAS_ERROR(r) ) {
+		signedfile_free(signedfile);
 		release_free(release);
-		(void)fclose(f);
-		return RET_ERRNO(e);
+		return r;
 	}
-	if( fclose(f) < 0 ) {
-		e = errno;
-		fprintf(stderr,"Error writing to %s: %d=$m!\n",
-				release->newreleasefilename,e);
-		release_free(release);
-		return RET_ERRNO(e);
-	}
-
-	if( distribution->signwith != NULL ) { 
-
-		release->signfilename = calc_dirconcat(release->dirofdist,"Release.gpg");
-		if( release->signfilename == NULL ) {
-			release_free(release);
-			return RET_ERROR_OOM;
-		}
-		release->newsignfilename = calc_dirconcat(release->dirofdist,"Release.gpg.new");
-		if( release->newsignfilename == NULL ) {
-			release_free(release);
-			return RET_ERROR_OOM;
-		}
-
-		r = signature_sign(distribution->signwith,
-				release->newreleasefilename,
-				release->newsignfilename);
-		if( RET_WAS_ERROR(r) ) {
-			release_free(release);
-			return r;
-		}
-	}
-
 	somethingwasdone = FALSE;
 	result = RET_OK;
 
@@ -1316,11 +1250,12 @@ retvalue release_write(/*@only@*/struct release *release, struct distribution *d
 					file->fullfinalfilename);
 			if( e < 0 ) {
 				e = errno;
-				fprintf(stderr,"Error moving %s to %s: %d=%m!",file->fulltemporaryfilename,file->fullfinalfilename,e);
+				fprintf(stderr,"Error moving %s to %s: %d=%m!\n",file->fulltemporaryfilename,file->fullfinalfilename,e);
 				r = RET_ERRNO(e);
 				/* after something was done, do not stop
 				 * but try to do as much as possible */
 				if( !somethingwasdone ) {
+					signedfile_free(signedfile);
 					release_free(release);
 					return r;
 				}
@@ -1329,40 +1264,21 @@ retvalue release_write(/*@only@*/struct release *release, struct distribution *d
 				somethingwasdone = TRUE;
 		}
 	}
-	if( release->newsignfilename != NULL && release->signfilename != NULL ) {
-		e = rename(release->newsignfilename,release->signfilename);
-		if( e < 0 ) {
-			e = errno;
-			fprintf(stderr,"Error moving %s to %s: %d=%m!",
-					release->newsignfilename,
-					release->signfilename,e);
-			r = RET_ERRNO(e);
-			/* after something was done, do not stop
-			 * but try to do as much as possible */
-			if( !somethingwasdone ) {
-				release_free(release);
-				return r;
-			}
-			RET_UPDATE(result,r);
-		}
+	r = signedfile_finalize(signedfile, &somethingwasdone);
+	signedfile_free(signedfile);
+	if( RET_WAS_ERROR(r) && !somethingwasdone ) {
+		release_free(release);
+		return r;
 	}
-	e = rename(release->newreleasefilename,release->releasefilename);
-	if( e < 0 ) {
-		e = errno;
-		fprintf(stderr,"Error moving %s to %s: %d=%m!",
-				release->newreleasefilename,
-				release->releasefilename,e);
-		r = RET_ERRNO(e);
-		RET_UPDATE(result,r);
-	}
-
-	/* now update the cache database, so we find those the next time */
-
-	r = storechecksums(release);
 	RET_UPDATE(result,r);
 
 	if( release->cachedb != NULL ) {
 	 	int dbret;
+
+		/* now update the cache database, so we find those the next time */
+		r = storechecksums(release);
+		RET_UPDATE(result,r);
+
 		/* check for an possible error in the cachedb,
 		 * release_free return no error */
 		dbret = release->cachedb->close(release->cachedb,0);
@@ -1377,3 +1293,15 @@ retvalue release_write(/*@only@*/struct release *release, struct distribution *d
 	return result;
 }
 
+retvalue release_mkdir(struct release *release, const char *relativedirectory) {
+	char *dirname;
+	retvalue r;
+
+	dirname = calc_dirconcat(release->dirofdist,relativedirectory);
+	if( dirname == NULL )
+		return RET_ERROR_OOM;
+	// TODO: in some far future, remember which dirs were created so that
+	r = dirs_make_recursive(dirname);
+	free(dirname);
+	return r;
+}
