@@ -1,7 +1,7 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2003,2004,2005,2006 Bernhard R. Link
+ *  Copyright (C) 2003,2004,2005,2006,2007 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as 
+ *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
@@ -31,6 +31,7 @@
 #include "dpkgversions.h"
 #include "override.h"
 #include "tracking.h"
+#include "signature.h"
 
 extern int verbose;
 
@@ -122,7 +123,7 @@ static retvalue parse_chunk(const char *chunk,/*@out@*/char **origdirectory,/*@o
 		if( !RET_IS_OK(r) ) {
 			return r;
 		}
-		if( verbose > 33 ) 
+		if( verbose > 33 )
 			fprintf(stderr,"got: %s\n",od);
 	}
 
@@ -131,7 +132,7 @@ static retvalue parse_chunk(const char *chunk,/*@out@*/char **origdirectory,/*@o
 
   	if( basefiles != NULL ) {
 		struct strlist filelines;
-  
+
 		r = chunk_getextralinelist(chunk,"Files",&filelines);
 		if( !RET_IS_OK(r) ) {
 			if( origdirectory != NULL )
@@ -148,7 +149,7 @@ static retvalue parse_chunk(const char *chunk,/*@out@*/char **origdirectory,/*@o
 				free(od);
   			return r;
 		}
-	} else 
+	} else
 		assert(md5sums == NULL); /* only together with basefiles */
 
 	if( origdirectory != NULL )
@@ -174,9 +175,9 @@ static inline retvalue calcnewcontrol(
 	}
 
 	directory = calc_sourcedir(component,package);
-	if( directory == NULL ) 
+	if( directory == NULL )
 		return RET_ERROR_OOM;
-	
+
 	r = calc_dirconcats(directory,basenames,filekeys);
 	if( RET_WAS_ERROR(r) ) {
 		free(directory);
@@ -197,18 +198,6 @@ static inline retvalue calcnewcontrol(
 	return RET_OK;
 }
 
-/* Get the files and their expected md5sums */
-retvalue sources_parse_getmd5sums(const char *chunk,struct strlist *basenames, struct strlist *md5sums) {
-	retvalue r;
-
-	r = parse_chunk(chunk,NULL,basenames,md5sums);
-	if( r == RET_NOTHING ) {
-		fprintf(stderr,"Does not look like source control: '%s'\n",chunk);
-		return RET_ERROR;
-	}
-	return r;
-}
-	
 retvalue sources_calcfilelines(const struct strlist *basenames,const struct strlist *md5sums,char **item) {
 	size_t len;
 	int i;
@@ -262,7 +251,7 @@ retvalue sources_getversion(UNUSED(struct target *t),const char *control,char **
 	}
 	return r;
 }
-	
+
 retvalue sources_getinstalldata(struct target *t,const char *packagename,UNUSED(const char *version),const char *chunk,char **control,struct strlist *filekeys,struct strlist *md5sums,struct strlist *origfiles) {
 	retvalue r;
 	char *origdirectory;
@@ -310,13 +299,13 @@ retvalue sources_getfilekeys(UNUSED(struct target *t),const char *chunk,struct s
 	char *origdirectory;
 	struct strlist basenames,mymd5sums;
 	retvalue r;
-	
+
 	if( md5sums != NULL )
 		r = parse_chunk(chunk,&origdirectory,&basenames,&mymd5sums);
 	else
 		r = parse_chunk(chunk,&origdirectory,&basenames,NULL);
 	if( r == RET_NOTHING ) {
-		//TODO: check if it is even text and do not print 
+		//TODO: check if it is even text and do not print
 		//of looking binary??
 		fprintf(stderr,"Does not look like source control: '%s'\n",chunk);
 		return RET_ERROR;
@@ -343,7 +332,7 @@ char *sources_getupstreamindex(UNUSED(struct target *target),const char *suite_f
 	return mprintf("dists/%s/%s/source/Sources.gz",suite_from,component_from);
 }
 
-retvalue sources_doreoverride(const struct alloverrides *ao,const char *packagename,const char *controlchunk,/*@out@*/char **newcontrolchunk) {
+retvalue sources_doreoverride(const struct distribution *distribution,const char *packagename,const char *controlchunk,/*@out@*/char **newcontrolchunk) {
 	const struct overrideinfo *o;
 	struct fieldtoadd *fields;
 	char *newchunk;
@@ -351,7 +340,7 @@ retvalue sources_doreoverride(const struct alloverrides *ao,const char *packagen
 	if( interrupted() )
 		return RET_ERROR_INTERUPTED;
 
-	o = override_search(ao->dsc, packagename);
+	o = override_search(distribution->overrides.dsc, packagename);
 	if( o == NULL )
 		return RET_NOTHING;
 
@@ -437,3 +426,157 @@ retvalue sources_getsourceandversion(UNUSED(struct target *t),const char *chunk,
 	*version = sourceversion;
 	return RET_OK;
 }
+
+/****************************************************************/
+
+static inline retvalue getvalue(const char *filename,const char *chunk,const char *field,char **value) {
+	retvalue r;
+
+	r = chunk_getvalue(chunk,field,value);
+	if( r == RET_NOTHING ) {
+		fprintf(stderr,"Missing '%s'-header in %s!\n",field,filename);
+		r = RET_ERROR;
+	}
+	return r;
+}
+
+static inline retvalue checkvalue(const char *filename,const char *chunk,const char *field) {
+	retvalue r;
+
+	r = chunk_checkfield(chunk,field);
+	if( r == RET_NOTHING ) {
+		fprintf(stderr,"Cannot find '%s'-header in %s!\n",field,filename);
+		r = RET_ERROR;
+	}
+	return r;
+}
+
+static inline retvalue getvalue_n(const char *chunk,const char *field,char **value) {
+	retvalue r;
+
+	r = chunk_getvalue(chunk,field,value);
+	if( r == RET_NOTHING ) {
+		*value = NULL;
+	}
+	return r;
+}
+
+retvalue sources_readdsc(struct dsc_headers *dsc, const char *filename, bool_t *broken) {
+	retvalue r;
+
+	r = signature_readsignedchunk(filename, filename, &dsc->control, NULL, NULL, broken);
+	if( RET_WAS_ERROR(r) ) {
+		return r;
+	}
+	if( verbose > 100 ) {
+		fprintf(stderr,"Extracted control chunk from '%s': '%s'\n",filename,dsc->control);
+	}
+
+	/* first look for fields that should be there */
+
+	r = chunk_getname(dsc->control,"Source",&dsc->name,FALSE);
+	if( r == RET_NOTHING ) {
+		fprintf(stderr,"Missing 'Source'-header in %s!\n",filename);
+		return RET_ERROR;
+	}
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	/* This is needed and cannot be ignored unless 
+	 * sources_complete is changed to not need it */
+	r = checkvalue(filename,dsc->control,"Format");
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	r = checkvalue(filename,dsc->control,"Maintainer");
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	/* only recommended, so ignore errors with this: */
+	(void) checkvalue(filename,dsc->control,"Standards-Version");
+
+	r = getvalue(filename,dsc->control,"Version",&dsc->version);
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	r = getvalue_n(dsc->control,SECTION_FIELDNAME,&dsc->section);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	r = getvalue_n(dsc->control,PRIORITY_FIELDNAME,&dsc->priority);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	r = parse_chunk(dsc->control,NULL,&dsc->basenames,&dsc->md5sums);
+	if( r == RET_NOTHING ) {
+		fprintf(stderr,"Missing 'Files'-header in %s!\n",filename);
+		return RET_ERROR;
+	}
+	return r;
+}
+
+void sources_done(struct dsc_headers *dsc) {
+	free(dsc->name);
+	free(dsc->version);
+	free(dsc->control);
+	strlist_done(&dsc->basenames);
+	strlist_done(&dsc->md5sums);
+	free(dsc->section);
+	free(dsc->priority);
+}
+
+retvalue sources_complete(const struct dsc_headers *dsc, const char *directory, const struct overrideinfo *override, const char *section, const char *priority, char **newcontrol) {
+	retvalue r;
+	struct fieldtoadd *name;
+	struct fieldtoadd *replace;
+	char *newchunk,*newchunk2;
+	char *newfilelines;
+
+	assert(section != NULL && priority != NULL);
+
+	/* first replace the "Source" with a "Package": */
+	name = addfield_new("Package",dsc->name,NULL);
+	if( name == NULL )
+		return RET_ERROR_OOM;
+	name = deletefield_new("Source",name);
+	if( name == NULL )
+		return RET_ERROR_OOM;
+	newchunk2  = chunk_replacefields(dsc->control,name,"Format");
+	addfield_free(name);
+	if( newchunk2 == NULL ) {
+		return RET_ERROR_OOM;
+	}
+
+	r = sources_calcfilelines(&dsc->basenames,&dsc->md5sums,&newfilelines);
+	if( RET_WAS_ERROR(r) ) {
+		free(newchunk2);
+		return RET_ERROR_OOM;
+	}
+	replace = addfield_new("Files",newfilelines,NULL);
+	if( replace != NULL )
+		replace = addfield_new("Directory",directory,replace);
+	if( replace != NULL )
+		replace = deletefield_new("Status",replace);
+	if( replace != NULL )
+		replace = addfield_new(SECTION_FIELDNAME,section,replace);
+	if( replace != NULL )
+		replace = addfield_new(PRIORITY_FIELDNAME,priority,replace);
+	if( replace != NULL )
+		replace = override_addreplacefields(override,replace);
+	if( replace == NULL ) {
+		free(newfilelines);
+		free(newchunk2);
+		return RET_ERROR_OOM;
+	}
+
+	newchunk  = chunk_replacefields(newchunk2,replace,"Files");
+	free(newfilelines);
+	free(newchunk2);
+	addfield_free(replace);
+	if( newchunk == NULL ) {
+		return RET_ERROR_OOM;
+	}
+
+	*newcontrol = newchunk;
+
+	return RET_OK;
+}
+
