@@ -1,5 +1,5 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2006 Bernhard R. Link
+ *  Copyright (C) 2006,2007 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -45,15 +45,20 @@
 int verbose=0;
 bool_t interrupted(void) {return FALSE;}
 
+static void about(bool_t help) NORETURN;
 static void about(bool_t help) {
 	fprintf(help?stdout:stderr,
 "modifychanges: Modify a Debian style .changes file\n"
-"Syntax: modifychanges <changesfile> <commands>\n"
+"Syntax: modifychanges [--create] <changesfile> <commands>\n"
 "Possible commands include:\n"
 " verify\n"
 " updatechecksums [<files to update>]\n"
 " includeallsources [<files to copy from .dsc to .changes>]\n"
-//" add <filenames>\n"
+" adddeb <.deb filenames>\n"
+" adddsc <.dsc filenames>\n"
+" addrawfile <filenames>\n"
+" add <filenames processed by filename suffix>\n"
+" setdistribution <distributions to list>\n"
 //" create <.dsc and .deb files to include>\n"
 );
 	if( help )
@@ -164,6 +169,11 @@ struct changes;
 static struct fileentry *add_fileentry(struct changes *c, const char *basename, size_t len, bool_t source);
 
 struct changes {
+	/* the filename of the .changes file */
+	char *filename;
+	/* directory of filename */
+	char *basedir;
+	/* Contents of the .changes file: */
 	char *name;
 	char *version;
 	char *maintainer;
@@ -181,12 +191,31 @@ struct changes {
 	bool_t modified;
 };
 
+static void fileentry_free(struct fileentry *f) {
+	if( f == NULL )
+		return;
+	free(f->basename);
+	free(f->fullfilename);
+	free(f->changesmd5sum);
+	free(f->realmd5sum);
+	free(f->section);
+	free(f->priority);
+	if( f->type == ft_DEB || f->type == ft_UDEB ) {
+		binaryfile_free(f->deb);
+	} else if( f->type == ft_DSC ) {
+		dscfile_free(f->dsc);
+	}
+	free(f);
+}
+
 static void changes_free(struct changes *c) {
 	unsigned int i;
 
 	if( c == NULL )
 		return;
 
+	free(c->filename);
+	free(c->basedir);
 	free(c->name);
 	free(c->version);
 	free(c->maintainer);
@@ -202,18 +231,7 @@ static void changes_free(struct changes *c) {
 	while( c->files ) {
 		struct fileentry *f = c->files;
 		c->files = f->next;
-		free(f->basename);
-		free(f->fullfilename);
-		free(f->changesmd5sum);
-		free(f->realmd5sum);
-		free(f->section);
-		free(f->priority);
-		if( f->type == ft_DEB || f->type == ft_UDEB ) {
-			binaryfile_free(f->deb);
-		} else if( f->type == ft_DSC ) {
-			dscfile_free(f->dsc);
-		}
-		free(f);
+		fileentry_free(f);
 	}
 	free(c);
 }
@@ -246,6 +264,87 @@ static struct fileentry *add_fileentry(struct changes *c, const char *basename,
 	return f;
 }
 
+static retvalue searchforfile(const char *changesdir, const char *basename,
+		const struct strlist *searchpath, char **result) {
+	int i; bool_t found;
+	char *fullname = calc_dirconcat(changesdir,basename);
+	if( fullname == NULL )
+		return RET_ERROR_OOM;
+
+	found = isregularfile(fullname);
+	i = 0;
+	while( !found && searchpath != NULL && i < searchpath->count ) {
+		free(fullname);
+		fullname = calc_dirconcat(searchpath->values[i],
+				basename);
+		if( fullname == NULL )
+			return RET_ERROR_OOM;
+		if( isregularfile(fullname) ) {
+			found = TRUE;
+			break;
+		}
+		i++;
+	}
+	if( found ) {
+		*result = fullname;
+		return RET_OK;
+	} else {
+		free(fullname);
+		return RET_NOTHING;
+	}
+}
+
+static retvalue findfile(const char *filename, const struct changes *c, const struct strlist *searchpath, char **result) {
+	char *fullfilename;
+
+	if( rindex(filename,'/') == NULL ) {
+		retvalue r;
+
+		r = searchforfile(c->basedir, filename, searchpath, &fullfilename);
+		if( !RET_IS_OK(r) )
+			return r;
+	} else {
+		if( !isregularfile(filename) )
+			return RET_NOTHING;
+		fullfilename = strdup(filename);
+		if( fullfilename == NULL ) {
+			return RET_ERROR_OOM;
+		}
+	}
+	*result = fullfilename;
+	return RET_OK;
+}
+
+static retvalue add_file(struct changes *c, /*@only@*/char *basename, /*@only@*/char *fullfilename, enum filetype type, struct fileentry **file) {
+	size_t basenamelen;
+	struct fileentry **fp = &c->files;
+	struct fileentry *f;
+
+	basenamelen = strlen(basename);
+
+	while( (f=*fp) != NULL ) {
+		if( f->namelen == basenamelen &&
+				strncmp(basename,f->basename,basenamelen) == 0 ) {
+			*file = f;
+			return RET_NOTHING;
+		}
+		fp = &f->next;
+	}
+	assert( f == NULL );
+	f = calloc(1,sizeof(struct fileentry));
+	if( f == NULL ) {
+		return RET_ERROR_OOM;
+	}
+	f->basename = basename;
+	f->namelen = basenamelen;
+	f->fullfilename = fullfilename;
+	f->type = type;
+
+	*fp = f;
+	*file = f;
+	return RET_OK;
+}
+
 
 static struct binary *get_binary(struct changes *c, const char *p, size_t len) {
 	unsigned int j;
@@ -380,17 +479,15 @@ static retvalue parse_changes_files(struct changes *c, struct strlist *tmp) {
 	return RET_OK;
 }
 
-static retvalue parse_dsc(struct fileentry *dscfile, struct changes *changes) {
+static retvalue read_dscfile(const char *fullfilename, struct dscfile **dsc) {
 	struct dscfile *n;
 	struct strlist tmp;
 	retvalue r;
 
-	if( dscfile->fullfilename == NULL )
-		return RET_NOTHING;
 	n = calloc(1,sizeof(struct dscfile));
 	if( n == NULL )
 		return RET_ERROR_OOM;
-	r = signature_readsignedchunk(dscfile->fullfilename,
+	r = signature_readsignedchunk(fullfilename, fullfilename,
 			&n->controlchunk, &n->validkeys, &n->keys, NULL);
 	assert( r != RET_NOTHING );
 	// TODO: can this be ignored sometimes?
@@ -431,6 +528,7 @@ static retvalue parse_dsc(struct fileentry *dscfile, struct changes *changes) {
 		int i,j = 0;
 		n->files = calloc(tmp.count, sizeof(struct sourcefile));
 		for( i = 0 ; i < tmp.count ; i++ ) {
+			n->files[j].file = NULL;
 			r = calc_parsefileline(tmp.values[i],
 					&n->files[j].basename,
 					&n->files[j].expectedmd5sum);
@@ -440,17 +538,8 @@ static retvalue parse_dsc(struct fileentry *dscfile, struct changes *changes) {
 				return r;
 			}
 			if( RET_IS_OK(r) ) {
-				n->files[j].file = add_fileentry(changes,
-						n->files[j].basename,
-						strlen(n->files[j].basename),
-						TRUE);
 				j++;
 				n->filecount = j;
-				if( n->files[j-1].file == NULL ) {
-					strlist_done(&tmp);
-					dscfile_free(n);
-					return RET_ERROR_OOM;
-				}
 			}
 		}
 		strlist_done(&tmp);
@@ -458,6 +547,31 @@ static retvalue parse_dsc(struct fileentry *dscfile, struct changes *changes) {
 		dscfile_free(n);
 		return r;
 	}
+	*dsc = n;
+	return RET_OK;
+}
+
+static retvalue parse_dsc(struct fileentry *dscfile, struct changes *changes) {
+	struct dscfile *n;
+	retvalue r;
+	int i;
+
+	if( dscfile->fullfilename == NULL )
+		return RET_NOTHING;
+	r = read_dscfile(dscfile->fullfilename, &n);
+	assert( r != RET_NOTHING );
+	if( RET_WAS_ERROR(r) )
+		return r;
+	for( i =  0 ; i < n->filecount ; i++ ) {
+		n->files[i].file = add_fileentry(changes,
+				n->files[i].basename,
+				strlen(n->files[i].basename),
+				TRUE);
+		if( n->files[i].file == NULL ) {
+			dscfile_free(n);
+			return RET_ERROR_OOM;
+		}
+	}
 	dscfile->dsc = n;
 	return RET_OK;
 }
@@ -522,17 +636,15 @@ static retvalue write_dsc_file(struct fileentry *dscfile, unsigned int flags) {
 	return RET_OK;
 }
 
-static retvalue parse_deb(struct fileentry *debfile, struct changes *changes) {
+static retvalue read_binaryfile(const char *fullfilename, struct binaryfile **result) {
 	retvalue r;
 	struct binaryfile *n;
 
-	if( debfile->fullfilename == NULL )
-		return RET_NOTHING;
 	n = calloc(1,sizeof(struct binaryfile));
 	if( n == NULL )
 		return RET_ERROR_OOM;
 
-	r = extractcontrol(&n->controlchunk, debfile->fullfilename);
+	r = extractcontrol(&n->controlchunk, fullfilename);
 	if( !RET_IS_OK(r)) {
 		free(n);
 		if( r == RET_ERROR_OOM )
@@ -582,6 +694,19 @@ static retvalue parse_deb(struct fileentry *debfile, struct changes *changes) {
 		binaryfile_free(n);
 		return r;
 	}
+	*result = n;
+	return RET_OK;
+}
+
+static retvalue parse_deb(struct fileentry *debfile, struct changes *changes) {
+	retvalue r;
+	struct binaryfile *n;
+
+	if( debfile->fullfilename == NULL )
+		return RET_NOTHING;
+	r = read_binaryfile(debfile->fullfilename, &n);
+	if( !RET_IS_OK(r) )
+		return r;
 	if( n->name != NULL ) {
 		n->binary = get_binary(changes, n->name, strlen(n->name));
 		if( n->binary == NULL ) {
@@ -607,33 +732,20 @@ static retvalue processfiles(const char *changesfilename, struct changes *change
 		return r;
 
 	for( file = changes->files; file != NULL ; file = file->next ) {
-		int i; bool_t found;
-		char *fullname = calc_dirconcat(dir,file->basename);
-		if( fullname == NULL )
-			return RET_ERROR_OOM;
 		assert( file->fullfilename == NULL );
 
-		found = isregularfile(fullname);
-		i = 0;
-		while( !found && searchpath != NULL && i < searchpath->count ) {
-			free(fullname);
-			fullname = calc_dirconcat(searchpath->values[i],
-					file->basename);
-			if( fullname == NULL )
-				return RET_ERROR_OOM;
-		}
+		r = searchforfile(dir, file->basename, searchpath, &file->fullfilename);
 
-		r = RET_NOTHING;
-		if( found ) {
-			file->fullfilename = fullname;
+		if( RET_IS_OK(r) ) {
 			if( file->type == ft_DSC )
 				r = parse_dsc(file,changes);
 			else if( file->type == ft_DEB || file->type == ft_UDEB )
 				r = parse_deb(file,changes);
-			if( RET_WAS_ERROR(r) )
+			if( RET_WAS_ERROR(r) ) {
+				free(dir);
 				return r;
-		} else
-			free(fullname);
+			}
+		}
 
 		if( r == RET_NOTHING ) {
 			/* apply heuristics when not readable */
@@ -646,8 +758,10 @@ static retvalue processfiles(const char *changesfilename, struct changes *change
 						file->basename[len] != '\0' )
 					len++;
 				b = get_binary(changes, file->basename, len);
-				if( b == NULL )
+				if( b == NULL ) {
+					free(dir);
 					return RET_ERROR_OOM;
+				}
 				b->uncheckable = TRUE;
 			}
 		}
@@ -664,6 +778,13 @@ static retvalue parse_changes(const char *changesfile, const char *chunk, struct
 	struct changes *n = calloc(1,sizeof(struct changes));
 	if( n == NULL )
 		return RET_ERROR_OOM;
+	n->filename = strdup(changesfile);
+	if( n->filename == NULL ) {
+		changes_free(n);
+		return RET_ERROR_OOM;
+	}
+	r = dirs_getdirectory(changesfile, &n->basedir);
+	R;
 	// TODO: do getname here? trim spaces?
 	r = chunk_getvalue(chunk, "Source", &n->name);
 	R;
@@ -691,7 +812,7 @@ static retvalue parse_changes(const char *changesfile, const char *chunk, struct
 		fprintf(stderr, "Missing 'Maintainer:' field in %s!\n", changesfile);
 		n->maintainer = NULL;
 	}
-	r = chunk_getwordlist(chunk, "Binary", &tmp);
+	r = chunk_getuniqwordlist(chunk, "Binary", &tmp);
 	R;
 	if( r == RET_NOTHING ) {
 		n->binaries = NULL;
@@ -743,6 +864,7 @@ static retvalue parse_changes(const char *changesfile, const char *chunk, struct
 #define CHANGES_WRITE_VERSION		0x08
 #define CHANGES_WRITE_ARCHITECTURES	0x10
 #define CHANGES_WRITE_MAINTAINER 	0x20
+#define CHANGES_WRITE_DISTRIBUTIONS 	0x40
 #define CHANGES_WRITE_ALL 	      0xFFFF
 
 static retvalue write_changes_file(const char *changesfilename,struct changes *c, unsigned int flags) {
@@ -831,6 +953,22 @@ static retvalue write_changes_file(const char *changesfilename,struct changes *c
 	cef = cef_newfield("Distribution", CEF_KEEP, CEF_EARLY, 0, cef);
 	if( cef == NULL )
 			return RET_ERROR_OOM;
+	if( c->distributions.count > 0 ) {
+		if( flagset(CHANGES_WRITE_DISTRIBUTIONS) )
+			cef = cef_newfield("Distribution", CEF_ADD,
+					CEF_EARLY, 0, cef);
+		else
+			cef = cef_newfield("Distribution", CEF_ADDMISSED,
+					CEF_EARLY, 0, cef);
+		if( cef == NULL )
+			return RET_ERROR_OOM;
+		cef_setwordlist(cef, &c->distributions);
+	} else if( flagset(CHANGES_WRITE_DISTRIBUTIONS) ) {
+		cef = cef_newfield("Distribution", CEF_DELETE,
+				CEF_EARLY, 0, cef);
+		if( cef == NULL )
+			return RET_ERROR_OOM;
+	}
 	if( c->version != NULL ) {
 		if( flagset(CHANGES_WRITE_VERSION) )
 			cef = cef_newfield("Version", CEF_ADD,
@@ -932,7 +1070,7 @@ static retvalue write_changes_file(const char *changesfilename,struct changes *c
 	}
 	cef_setdata(cef, "1.7");
 
-	r = chunk_edit(c->control, &control, &controllen, cef);
+	r = chunk_edit((c->control==NULL)?"":c->control, &control, &controllen, cef);
 	strlist_done(&binaries);
 	cef_free(cef);
 	if( RET_WAS_ERROR(r) )
@@ -992,7 +1130,8 @@ static void verify_sourcefile_md5sums(struct sourcefile *f, const char *dscfile)
 		return;
 	}
 	if( strcmp(f->expectedmd5sum, f->file->realmd5sum) != 0 ) {
-		if( strcmp(f->expectedmd5sum, f->file->changesmd5sum) == 0 )
+		if( f->file->changesmd5sum != NULL &&
+		    strcmp(f->expectedmd5sum, f->file->changesmd5sum) == 0 )
 			fprintf(stderr,
 "ERROR: '%s' lists the same wrong md5sum for '%s' like the .changes file!\n",
 				dscfile, f->basename);
@@ -1006,6 +1145,8 @@ static void verify_sourcefile_md5sums(struct sourcefile *f, const char *dscfile)
 
 static void verify_binary_name(const char *basename, const char *name, const char *version, const char *architecture, enum filetype type) {
 	size_t nlen, vlen, alen;
+	const char *versionwithoutepoch;
+
 	if( name == NULL )
 		return;
 	nlen = strlen(name);
@@ -1017,8 +1158,13 @@ static void verify_binary_name(const char *basename, const char *name, const cha
 	}
 	if( version == NULL )
 		return;
-	vlen = strlen(version);
-	if( strncmp(basename+nlen+1, version, vlen) != 0
+	versionwithoutepoch = strchr(version, ':');
+	if( versionwithoutepoch == NULL )
+		versionwithoutepoch = version;
+	else
+		versionwithoutepoch++;
+	vlen = strlen(versionwithoutepoch);
+	if( strncmp(basename+nlen+1, versionwithoutepoch, vlen) != 0
 	|| basename[nlen+1+vlen] != '_' ) {
 		fprintf(stderr,
 "ERROR: '%s' does not start with '%s_%s_' as expected!\n",
@@ -1032,7 +1178,8 @@ static void verify_binary_name(const char *basename, const char *name, const cha
 	|| strcmp(basename+nlen+1+vlen+1+alen, typesuffix[type]) != 0 )
 		fprintf(stderr,
 "ERROR: '%s' is not called '%s_%s_%s%s' as expected!\n",
-			basename, name, version, architecture, typesuffix[type]);
+			basename, name, versionwithoutepoch,
+			architecture, typesuffix[type]);
 }
 
 static retvalue verify(const char *changesfilename, struct changes *changes) {
@@ -1160,6 +1307,14 @@ static retvalue verify(const char *changesfilename, struct changes *changes) {
 				}
 			}
 		}
+		if( version != NULL ) {
+			const char *colon = strchr(version, ':');
+			if( colon != NULL ) {
+				colon++;
+				versionlen -= (colon-version);
+				version = colon;
+			}
+		}
 		if( name != NULL && version != NULL ) {
 			if( *p != '_'
 			|| (size_t)(p-file->basename) != namelen || l-4 != versionlen
@@ -1632,7 +1787,512 @@ static retvalue includeallsources(const char *changesfilename, struct changes *c
 		return RET_NOTHING;
 }
 
-static int execute_command(int argc, char **argv, const char *changesfilename, bool_t file_exists, struct changes *changesdata) {
+static retvalue adddsc(struct changes *c, const char *dscfilename) {
+	retvalue r;
+	struct fileentry *f;
+	struct dscfile *dsc;
+	char *fullfilename, *basename;
+	int i;
+
+	r = findfile(dscfilename, c, NULL, &fullfilename);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	if( r == RET_NOTHING ) {
+		fprintf(stderr, "Cannot find '%s'!\n", dscfilename);
+		return RET_ERROR_MISSING;
+	}
+	r = read_dscfile(fullfilename, &dsc);
+	if( r == RET_NOTHING ) {
+		fprintf(stderr, "Error reading '%s'!\n", fullfilename);
+		r = RET_ERROR;
+	}
+	if( RET_WAS_ERROR(r) ) {
+		free(fullfilename);
+		return r;
+	}
+	if( dsc->name == NULL || dsc->version == NULL ) {
+		if( dsc->name == NULL )
+			fprintf(stderr, "Could not extract name of '%s'!\n",
+					fullfilename);
+		else
+			fprintf(stderr, "Could not extract version of '%s'!\n",
+					fullfilename);
+		dscfile_free(dsc);
+		free(fullfilename);
+		return RET_ERROR;
+	}
+	if( c->name != NULL ) {
+		if( strcmp(c->name, dsc->name) != 0 ) {
+			fprintf(stderr,
+"ERROR: '%s' lists source '%s' while '%s' already is '%s'!\n",
+					fullfilename, dsc->name,
+					c->filename, c->name);
+			dscfile_free(dsc);
+			free(fullfilename);
+			return RET_ERROR;
+		}
+	} else {
+		c->name = strdup(dsc->name);
+		if( c->name == NULL ) {
+			dscfile_free(dsc);
+			free(fullfilename);
+			return RET_ERROR_OOM;
+		}
+	}
+	if( c->version != NULL ) {
+		if( strcmp(c->version, dsc->version) != 0 )
+			fprintf(stderr,
+"WARNING: '%s' lists version '%s' while '%s' already lists '%s'!\n",
+					fullfilename, dsc->version,
+					c->filename, c->version);
+	} else {
+		c->version = strdup(dsc->version);
+		if( c->version == NULL ) {
+			dscfile_free(dsc);
+			free(fullfilename);
+			return RET_ERROR_OOM;
+		}
+	}
+	// TODO: make sure if the .changes name/version are modified they will
+	// also be written...
+	basename = calc_source_basename(dsc->name, dsc->version);
+	if( basename == NULL ) {
+		dscfile_free(dsc);
+		free(fullfilename);
+		return RET_ERROR_OOM;
+	}
+
+	// TODO: add rename/copy option to be activated when old and new
+	// basename differ
+
+	r = add_file(c, basename, fullfilename, ft_DSC, &f);
+	if( RET_WAS_ERROR(r) ) {
+		dscfile_free(dsc);
+		free(fullfilename);
+		free(basename);
+		return r;
+	}
+	if( r == RET_NOTHING ) {
+		fprintf(stderr, "ERROR: '%s' already contains a file of the same name!\n", c->filename);
+		dscfile_free(dsc);
+		free(fullfilename);
+		free(basename);
+		// TODO: check instead if it is already the same...
+		return RET_ERROR;
+	}
+	/* f owns dsc, fullfilename and basename now */
+	f->dsc = dsc;
+
+	/* now include the files needed by this */
+	for( i =  0 ; i < dsc->filecount ; i++ ) {
+		struct fileentry *file;
+		const char *basefilename = dsc->files[i].basename;
+		const char *md5sum = dsc->files[i].expectedmd5sum;
+
+		file = add_fileentry(c, basefilename,
+				strlen(basefilename),
+				TRUE);
+		if( file == NULL )
+			return RET_ERROR_OOM;
+		dsc->files[i].file = file;
+		/* make them appear in the .changes file if not there: */
+		if( file->changesmd5sum == NULL ) {
+			file->changesmd5sum = strdup(md5sum);
+			if( file->changesmd5sum == NULL )
+				return RET_ERROR_OOM;
+		} // TODO: otherwise warn if not the same
+	}
+
+	c->modified = TRUE;
+	r = md5sum_read(f->fullfilename, &f->realmd5sum);
+	if( RET_WAS_ERROR(r) ) {
+		return r;
+	}
+	f->changesmd5sum = strdup(f->realmd5sum);
+	if( f->changesmd5sum == NULL ) {
+		return RET_ERROR_OOM;;
+	}
+	/* for a "extended" dsc with section or priority or
+	 * for the future code for parsing .diff and .tar.gz  */
+	if( dsc->section != NULL ) {
+		free(f->section);
+		f->section = strdup(dsc->section);
+		if( f->section == NULL )
+			return RET_ERROR_OOM;
+	}
+	if( dsc->priority != NULL ) {
+		free(f->priority);
+		f->priority = strdup(dsc->priority);
+		if( f->priority == NULL )
+			return RET_ERROR_OOM;
+	}
+	/* update information in the main .changes file if not there already */
+	if( c->maintainer == NULL && dsc->maintainer != NULL ) {
+		c->maintainer = strdup(dsc->maintainer);
+		if( c->maintainer == NULL )
+			return RET_ERROR_OOM;
+	}
+	if( !strlist_in(&c->architectures, "source") ) {
+		r = strlist_add_dup(&c->architectures, "source");
+		if( RET_WAS_ERROR(r) )
+			return r;
+	}
+	return RET_OK;
+}
+
+static retvalue adddscs(const char *changesfilename, struct changes *c, int argc, char **argv) {
+	if( argc <= 0 ) {
+		fprintf(stderr, "Filenames of .dsc files to include expected!\n");
+		return RET_ERROR;
+	}
+	while( argc > 0 ) {
+		retvalue r = adddsc(c, argv[0]);
+		if( RET_WAS_ERROR(r) )
+			return r;
+		argc--; argv++;
+	}
+	if( c->modified ) {
+		return write_changes_file(changesfilename, c,
+				CHANGES_WRITE_ALL);
+	} else
+		return RET_NOTHING;
+}
+
+static retvalue adddeb(struct changes *c, const char *debfilename) {
+	retvalue r;
+	struct fileentry *f;
+	struct binaryfile *deb;
+	const char *packagetype;
+	enum filetype type;
+	char *fullfilename, *basename;
+
+	r = findfile(debfilename, c, NULL, &fullfilename);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	if( r == RET_NOTHING ) {
+		fprintf(stderr, "Cannot find '%s'!\n", debfilename);
+		return RET_ERROR_MISSING;
+	}
+	r = read_binaryfile(fullfilename, &deb);
+	if( r == RET_NOTHING ) {
+		fprintf(stderr, "Error reading '%s'!\n", fullfilename);
+		r = RET_ERROR;
+	}
+	if( RET_WAS_ERROR(r) ) {
+		free(fullfilename);
+		return r;
+	}
+	// TODO: check if there are other things but the name to distinguish them
+	if( strlen(fullfilename) > 5 &&
+			strcmp(fullfilename+strlen(fullfilename)-5,".udeb") == 0 ) {
+		packagetype = "udeb";
+		type = ft_UDEB;
+	} else {
+		packagetype = "deb";
+		type = ft_DEB;
+	}
+	if( deb->name == NULL || deb->version == NULL || deb->architecture == NULL ) {
+		if( deb->name == NULL )
+			fprintf(stderr, "Could not extract packagename of '%s'!\n",
+					fullfilename);
+		else if( deb->version == NULL )
+			fprintf(stderr, "Could not extract version of '%s'!\n",
+					fullfilename);
+		else
+			fprintf(stderr, "Could not extract architecture of '%s'!\n",
+					fullfilename);
+		binaryfile_free(deb);
+		free(fullfilename);
+		return RET_ERROR;
+	}
+	if( c->name != NULL ) {
+		const char *sourcename;
+		if( deb->sourcename != NULL )
+			sourcename = deb->sourcename;
+		else
+			sourcename = deb->name;
+		if( strcmp(c->name,sourcename) != 0 ) {
+			fprintf(stderr,
+"ERROR: '%s' lists source '%s' while '%s' already is '%s'!\n",
+					fullfilename, sourcename,
+					c->filename, c->name);
+			binaryfile_free(deb);
+			free(fullfilename);
+			return RET_ERROR;
+		}
+	} else {
+		if( deb->sourcename != NULL )
+			c->name = strdup(deb->sourcename);
+		else
+			c->name = strdup(deb->name);
+		if( c->name == NULL ) {
+			binaryfile_free(deb);
+			free(fullfilename);
+			return RET_ERROR_OOM;
+		}
+	}
+	if( c->version != NULL ) {
+		const char *sourceversion;
+		if( deb->sourceversion != NULL )
+			sourceversion = deb->sourceversion;
+		else
+			sourceversion = deb->version;
+		if( strcmp(c->version,sourceversion) != 0 )
+			fprintf(stderr,
+"WARNING: '%s' lists source version '%s' while '%s' already lists '%s'!\n",
+					fullfilename, sourceversion,
+					c->filename, c->version);
+	} else {
+		if( deb->sourceversion != NULL )
+			c->version = strdup(deb->sourceversion);
+		else
+			c->version = strdup(deb->version);
+		if( c->version == NULL ) {
+			binaryfile_free(deb);
+			free(fullfilename);
+			return RET_ERROR_OOM;
+		}
+	}
+	// TODO: make sure if the .changes name/version are modified they will
+	// also be written...
+	basename = calc_binary_basename(deb->name, deb->version,
+	                                deb->architecture, packagetype);
+	if( basename == NULL ) {
+		binaryfile_free(deb);
+		free(fullfilename);
+		return RET_ERROR_OOM;
+	}
+
+	// TODO: add rename/copy option to be activated when old and new
+	// basename differ
+
+	r = add_file(c, basename, fullfilename, type, &f);
+	if( RET_WAS_ERROR(r) ) {
+		binaryfile_free(deb);
+		free(fullfilename);
+		free(basename);
+		return r;
+	}
+	if( r == RET_NOTHING ) {
+		fprintf(stderr, "ERROR: '%s' already contains a file of the same name!\n", c->filename);
+		binaryfile_free(deb);
+		free(fullfilename);
+		free(basename);
+		// TODO: check instead if it is already the same...
+		return RET_ERROR;
+	}
+	/* f owns deb, fullfilename and basename now */
+	f->deb = deb;
+	deb->binary = get_binary(c, deb->name, strlen(deb->name));
+	if( deb->binary == NULL ) {
+		return RET_ERROR_OOM;
+	}
+	deb->next = deb->binary->files;
+	deb->binary->files = deb;
+	deb->binary->missedinheader = FALSE;
+	c->modified = TRUE;
+	r = md5sum_read(f->fullfilename, &f->realmd5sum);
+	if( RET_WAS_ERROR(r) ) {
+		return r;
+	}
+	f->changesmd5sum = strdup(f->realmd5sum);
+	if( f->changesmd5sum == NULL ) {
+		return RET_ERROR_OOM;;
+	}
+	if( deb->shortdescription != NULL ) {
+		if( deb->binary->description == NULL ) {
+			deb->binary->description = strdup(deb->shortdescription);
+			deb->binary->missedinheader = FALSE;
+		} else if( strcmp(deb->binary->description,
+		                  deb->shortdescription) != 0 ) {
+			fprintf(stderr,
+"WARNING: '%s' already lists a different description for '%s' than contained in '%s'!\n",
+					c->filename, deb->name, fullfilename);
+		}
+	}
+	if( deb->section != NULL ) {
+		free(f->section);
+		f->section = strdup(deb->section);
+	}
+	if( deb->priority != NULL ) {
+		free(f->priority);
+		f->priority = strdup(deb->priority);
+	}
+	if( c->maintainer == NULL && deb->maintainer != NULL ) {
+		c->maintainer = strdup(deb->maintainer);
+	}
+	if( deb->architecture != NULL &&
+			!strlist_in(&c->architectures, deb->architecture) ) {
+		strlist_add_dup(&c->architectures, deb->architecture);
+	}
+	return RET_OK;
+}
+
+static retvalue adddebs(const char *changesfilename, struct changes *c, int argc, char **argv) {
+	if( argc <= 0 ) {
+		fprintf(stderr, "Filenames of .deb files to include expected!\n");
+		return RET_ERROR;
+	}
+	while( argc > 0 ) {
+		retvalue r = adddeb(c, argv[0]);
+		if( RET_WAS_ERROR(r) )
+			return r;
+		argc--; argv++;
+	}
+	if( c->modified ) {
+		return write_changes_file(changesfilename, c,
+				CHANGES_WRITE_ALL);
+	} else
+		return RET_NOTHING;
+}
+
+static retvalue addrawfile(struct changes *c, const char *filename) {
+	retvalue r;
+	struct fileentry *f;
+	char *fullfilename, *basefilename;
+	char *md5sum;
+
+	r = findfile(filename, c, NULL, &fullfilename);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	if( r == RET_NOTHING ) {
+		fprintf(stderr, "Cannot find '%s'!\n", filename);
+		return RET_ERROR_MISSING;
+	}
+	basefilename = strdup(dirs_basename(filename));
+	if( basefilename == NULL ) {
+		free(fullfilename);
+		return RET_ERROR_OOM;
+	}
+	r = md5sum_read(fullfilename, &md5sum);
+	if( RET_WAS_ERROR(r) ) {
+		free(fullfilename);
+		free(basefilename);
+		return r;
+	}
+	r = add_file(c, basefilename, fullfilename, ft_UNKNOWN, &f);
+	if( RET_WAS_ERROR(r) ) {
+		free(fullfilename);
+		free(basefilename);
+		free(md5sum);
+		return r;
+	}
+	if( r == RET_NOTHING ) {
+
+		assert( f != NULL );
+
+		if( f->changesmd5sum != NULL ) {
+			/* already listed in .changes */
+
+			if( strcmp(f->changesmd5sum, md5sum) != 0 ) {
+				fprintf(stderr, "ERROR: '%s' already contains a file with name '%s' but different size or md5sum!\n", c->filename, basefilename);
+				free(fullfilename);
+				free(basefilename);
+				free(md5sum);
+				return RET_ERROR;
+			}
+			printf("'%s' already lists '%s' with same md5sum. Doing nothing.\n", c->filename, basefilename);
+			free(fullfilename);
+			free(basefilename);
+			free(md5sum);
+			return RET_NOTHING;
+		} else {
+			/* file already expected by some other part (e.g. a .dsc) */
+
+			// TODO: find out whom this files belong to and warn if different
+			free(fullfilename);
+			free(basefilename);
+		}
+	} else {
+		// fullfilename and basefilename now belong to *f
+		basefilename = NULL;
+		fullfilename = NULL;
+	}
+
+	c->modified = TRUE;
+	assert( f->changesmd5sum == NULL );
+	f->changesmd5sum = md5sum;
+	md5sum = NULL;
+	if( f->realmd5sum == NULL )
+		f->realmd5sum = strdup(f->changesmd5sum);
+	if( f->realmd5sum == NULL ) {
+		return RET_ERROR_OOM;;
+	}
+	return RET_OK;
+}
+
+static retvalue addrawfiles(const char *changesfilename, struct changes *c, int argc, char **argv) {
+	if( argc <= 0 ) {
+		fprintf(stderr, "Filenames of files to add (without further parsing) expected!\n");
+		return RET_ERROR;
+	}
+	while( argc > 0 ) {
+		retvalue r = addrawfile(c, argv[0]);
+		if( RET_WAS_ERROR(r) )
+			return r;
+		argc--; argv++;
+	}
+	if( c->modified ) {
+		return write_changes_file(changesfilename, c,
+				CHANGES_WRITE_FILES);
+	} else
+		return RET_NOTHING;
+}
+
+static retvalue addfiles(const char *changesfilename, struct changes *c, int argc, char **argv) {
+	if( argc <= 0 ) {
+		fprintf(stderr, "Filenames of files to add expected!\n");
+		return RET_ERROR;
+	}
+	while( argc > 0 ) {
+		retvalue r;
+		const char *filename = argv[0];
+		size_t l = strlen(filename);
+
+		if( (l > 4 && strcmp(filename+l-4, ".deb") == 0) ||
+		    (l > 5 && strcmp(filename+l-5, ".udeb") == 0) )
+			r = adddeb(c, filename);
+		else if( (l > 4 && strcmp(filename+l-4, ".dsc") == 0) )
+			r = adddsc(c, filename);
+		else
+			r = addrawfile(c, argv[0]);
+		if( RET_WAS_ERROR(r) )
+			return r;
+		argc--; argv++;
+	}
+	if( c->modified ) {
+		return write_changes_file(changesfilename, c,
+				CHANGES_WRITE_ALL);
+	} else
+		return RET_NOTHING;
+}
+
+static retvalue setdistribution(const char *changesfilename, struct changes *c, int argc, char **argv) {
+	retvalue r;
+	struct strlist distributions;
+	int i;
+
+	if( argc <= 0 ) {
+		fprintf(stderr, "expected Distribution name to set!\n");
+		return RET_ERROR;
+	}
+	r = strlist_init_n(argc, &distributions);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	for( i = 0 ; i < argc ; i++ ) {
+		r = strlist_add_dup(&distributions, argv[i]);
+		if( RET_WAS_ERROR(r) ) {
+			strlist_done(&distributions);
+			return r;
+		}
+	}
+	strlist_done(&c->distributions);
+	strlist_move(&c->distributions, &distributions);
+	return write_changes_file(changesfilename, c,
+			CHANGES_WRITE_DISTRIBUTIONS);
+}
+
+static int execute_command(int argc, char **argv, const char *changesfilename, bool_t file_exists, bool_t create_file, struct changes *changesdata) {
 	const char *command = argv[0];
 	retvalue r;
 
@@ -1665,6 +2325,46 @@ static int execute_command(int argc, char **argv, const char *changesfilename, b
 					changesfilename);
 			r = RET_ERROR;
 		}
+	} else if( strcasecmp(command, "addrawfile") == 0 ) {
+		if( file_exists || create_file )
+			r = addrawfiles(changesfilename, changesdata, argc-1, argv+1);
+		else {
+			fprintf(stderr, "No such file '%s'!\n",
+					changesfilename);
+			r = RET_ERROR;
+		}
+	} else if( strcasecmp(command, "adddsc") == 0 ) {
+		if( file_exists || create_file )
+			r = adddscs(changesfilename, changesdata, argc-1, argv+1);
+		else {
+			fprintf(stderr, "No such file '%s'!\n",
+					changesfilename);
+			r = RET_ERROR;
+		}
+	} else if( strcasecmp(command, "adddeb") == 0 ) {
+		if( file_exists || create_file )
+			r = adddebs(changesfilename, changesdata, argc-1, argv+1);
+		else {
+			fprintf(stderr, "No such file '%s'!\n",
+					changesfilename);
+			r = RET_ERROR;
+		}
+	} else if( strcasecmp(command, "add") == 0 ) {
+		if( file_exists || create_file )
+			r = addfiles(changesfilename, changesdata, argc-1, argv+1);
+		else {
+			fprintf(stderr, "No such file '%s'!\n",
+					changesfilename);
+			r = RET_ERROR;
+		}
+	} else if( strcasecmp(command, "setdistribution") == 0 ) {
+		if( file_exists )
+			r = setdistribution(changesfilename, changesdata, argc-1, argv+1);
+		else {
+			fprintf(stderr, "No such file '%s'!\n",
+					changesfilename);
+			r = RET_ERROR;
+		}
 	} else {
 		fprintf(stderr, "Unknown command '%s'\n", command);
 		r = RET_ERROR;
@@ -1672,37 +2372,72 @@ static int execute_command(int argc, char **argv, const char *changesfilename, b
 	return r;
 }
 
+static retvalue splitpath(struct strlist *list, const char *path) {
+	retvalue r;
+	const char *next;
+
+	while( (next = index(path, ':')) != NULL ) {
+		if( next > path ) {
+			char *dir = strndup(path, next-path);
+			if( dir == NULL ) {
+				return RET_ERROR_OOM;
+			}
+			r = strlist_add(list, dir);
+			if( RET_WAS_ERROR(r) )
+				return r;
+		}
+		path = next+1;
+	}
+	return strlist_add_dup(list, path);
+}
+
 int main(int argc,char *argv[]) {
 	static const struct option longopts[] = {
 		{"help", no_argument, NULL, 'h'},
+		{"create", no_argument, NULL, 'C'},
 		{"ignore", required_argument, NULL, 'i'},
+		{"searchpath", required_argument, NULL, 's'},
 		{NULL, 0, NULL, 0},
 	};
 	int c;
 	const char *changesfilename;
 	bool_t file_exists;
+	bool_t create_file = FALSE;
 	struct strlist validkeys,keys;
+	struct strlist searchpath;
 	struct changes *changesdata;
 	retvalue r;
 
+	strlist_init(&searchpath);
 	init_ignores();
 
 
-	while( (c = getopt_long(argc,argv,"+hi:",longopts,NULL)) != -1 ) {
+	while( (c = getopt_long(argc,argv,"+hi:s:",longopts,NULL)) != -1 ) {
 		switch( c ) {
 			case 'h':
 				about(TRUE);
 			case 'i':
 				set_ignore(optarg,FALSE,CONFIG_OWNER_CMDLINE);
 				break;
+			case 'C':
+				create_file = TRUE;
+				break;
+			case 's':
+				r = splitpath(&searchpath, optarg);
+				if( RET_WAS_ERROR(r) ) {
+					if( r == RET_ERROR_OOM )
+						fprintf(stderr, "Out of memory!\n");
+					exit(EXIT_FAILURE);
+				}
+				break;
 		}
 	}
-	if( argc <= 2 ) {
+	if( argc - optind < 2 ) {
 		about(FALSE);
 	}
 	signature_init(FALSE);
 
-	changesfilename = argv[1];
+	changesfilename = argv[optind];
 	if( strcmp(changesfilename,"-") != 0 && !endswith(changesfilename,".changes")
 			&& !IGNORING_(extension,
 				"first argument does not ending with '.changes'\n") )
@@ -1711,33 +2446,40 @@ int main(int argc,char *argv[]) {
 	if( file_exists ) {
 		char *changes;
 
-		r = signature_readsignedchunk(changesfilename, &changes, &validkeys, &keys, NULL);
+		r = signature_readsignedchunk(changesfilename, changesfilename,
+				&changes, &validkeys, &keys, NULL);
 		if( !RET_IS_OK(r) ) {
 			signatures_done();
 			if( r == RET_ERROR_OOM )
 				fprintf(stderr, "Out of memory!\n");
 			exit(EXIT_FAILURE);
 		}
-		r = parse_changes(changesfilename, changes, &changesdata, NULL);
+		r = parse_changes(changesfilename, changes, &changesdata, &searchpath);
 		if( RET_IS_OK(r) )
 			changesdata->control = changes;
-		else
+		else {
 			free(changes);
+			changesdata = NULL;
+		}
 	} else {
 		strlist_init(&keys);
 		strlist_init(&validkeys);
 		changesdata = calloc(1,sizeof(struct changes));
-		if( changesdata == NULL )
+		if( changesdata != NULL )
+			changesdata->filename = strdup(changesfilename);
+		if( changesdata == NULL || changesdata->filename == NULL )
 			r = RET_ERROR_OOM;
 		else {
-			r = RET_OK;
+			r = dirs_getdirectory(changesfilename,
+					&changesdata->basedir);
 		}
 	}
 
 	if( !RET_WAS_ERROR(r) ) {
-		argc -= 2;
-		argv += 2;
-		r = execute_command(argc,argv, changesfilename, file_exists, changesdata);
+		argc -= (optind+1);
+		argv += (optind+1);
+		r = execute_command(argc, argv, changesfilename,
+		                    file_exists, create_file, changesdata);
 	}
 	changes_free(changesdata);
 	strlist_done(&keys);
