@@ -1,5 +1,5 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2005,2006 Bernhard R. Link
+ *  Copyright (C) 2005,2006,2007 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -164,6 +164,7 @@ retvalue trackedpackage_addfilekey(trackingdb tracks,struct trackedpackage *pkg,
 
 	for( i = 0 ; i < pkg->filekeys.count ; i++ ) {
 		if( strcmp(pkg->filekeys.values[i],filekey) == 0 ) {
+			free(filekey);
 			if( pkg->filetypes[i] != ft ) {
 				fprintf(stderr,"Filekey '%s' already registered for '%s_%s' as type '%c' is tried to be reregistered as type '%c'!\n",filekey,pkg->sourcename,pkg->sourceversion,pkg->filetypes[i],ft);
 				return RET_ERROR;
@@ -237,7 +238,7 @@ static inline retvalue trackedpackage_removefilekey(trackingdb tracks,struct tra
 				pkg->refcounts[i]--;
 			} else
 				fprintf(stderr,"Warning: tracking database of %s has inconsistent refcounts of %s_%s.\n",tracks->codename,pkg->sourcename,pkg->sourceversion);
-				
+
 			return RET_OK;
 		}
 	}
@@ -273,7 +274,7 @@ void trackedpackage_free(struct trackedpackage *pkg) {
 
 static inline int parsenumber(const char **d,size_t *s) {
 	int count;
-	
+
 	count = 0;
 	do {
 		if( **d < '0' || **d > '7' )
@@ -293,7 +294,7 @@ static retvalue tracking_new(trackingdb t,const char *sourcename,const char *ver
 	struct trackedpackage *p;
 	assert( pkg != NULL && sourcename != NULL && version != NULL );
 
-	printf("[tracking_new %s %s %s]\n",t->codename,sourcename,version);
+//	printf("[tracking_new %s %s %s]\n",t->codename,sourcename,version);
 	p = calloc(1,sizeof(struct trackedpackage));
 	if( p == NULL )
 		return RET_ERROR_OOM;
@@ -357,7 +358,7 @@ static inline retvalue parsedata(const char *name,const char *version,size_t ver
 		retvalue r;
 
 		if( ((p->filekeys.count)&31) == 0 ) {
-			enum filetype *n = realloc(p->filetypes, 
+			enum filetype *n = realloc(p->filetypes,
 				(p->filekeys.count+32)*sizeof(enum filetype));
 			if( n == NULL ) {
 				trackedpackage_free(p);
@@ -404,7 +405,24 @@ static inline retvalue parsedata(const char *name,const char *version,size_t ver
 	return RET_OK;
 }
 
-	
+static inline retvalue parseunknowndata(DBT key,DBT data,/*@out@*/struct trackedpackage **pkg) {
+	size_t versionlen;
+	const char *sep;
+
+	if( ((const char*)key.data)[key.size-1] != '\0' ) {
+		fprintf(stderr, "Tracking database corrupted! (unterminated key name!)\n");
+		return RET_ERROR;
+	}
+	sep = memchr(data.data, '\0', data.size);
+	if( sep == NULL ) {
+		fprintf(stderr, "Tracking database corrupted! (unterminated data!)\n");
+		return RET_ERROR;
+	}
+	versionlen = (sep - (const char*)data.data);
+	return parsedata(key.data, data.data, versionlen, data, pkg);
+}
+
+
 static retvalue tracking_get(trackingdb t,const char *sourcename,const char *version,/*@out@*/struct trackedpackage **pkg) {
 	int dbret;
 	DBT key,data;
@@ -414,7 +432,7 @@ static retvalue tracking_get(trackingdb t,const char *sourcename,const char *ver
 
 
 	assert( pkg != NULL && sourcename != NULL && version != NULL );
-	printf("[tracking_get %s %s %s]\n",t->codename,sourcename,version);
+//	printf("[tracking_get %s %s %s]\n",t->codename,sourcename,version);
 
 	cursor = NULL;
 	if( (dbret = t->db->cursor(t->db,NULL,&cursor,0)) != 0 ) {
@@ -434,7 +452,7 @@ static retvalue tracking_get(trackingdb t,const char *sourcename,const char *ver
 		}
 	}
 
-	printf("[tracking_get found %s %s %s]\n",t->codename,sourcename,version);
+//	printf("[tracking_get found %s %s %s]\n",t->codename,sourcename,version);
 	/* we have found it, now parse it */
 	r = parsedata(sourcename,version,versionlen,data,pkg);
 	(void)cursor->c_close(cursor);
@@ -514,8 +532,8 @@ static retvalue tracking_saveonly(trackingdb t, struct trackedpackage *pkg) {
 	retvalue r;
 
 	assert( pkg != NULL );
-	printf("[tracking_save %s %s %s]\n", t->codename,
-			pkg->sourcename, pkg->sourceversion);
+//	printf("[tracking_save %s %s %s]\n", t->codename,
+//			pkg->sourcename, pkg->sourceversion);
 
 	SETDBT(key,pkg->sourcename);
 
@@ -549,10 +567,10 @@ static retvalue tracking_saveonly(trackingdb t, struct trackedpackage *pkg) {
 				t->db->err(t->db, dbret, "tracking_save(delete) dberror:");
 				return RET_DBERR(dbret);
 			} else {
-				printf("[tracking: removed %s_%s from %s.]\n",
-						pkg->sourcename,
-						pkg->sourceversion,
-						t->codename);
+//				printf("[tracking: removed %s_%s from %s.]\n",
+//						pkg->sourcename,
+//						pkg->sourceversion,
+//						t->codename);
 				return RET_OK;
 			}
 		}
@@ -587,6 +605,7 @@ static retvalue tracking_saveonly(trackingdb t, struct trackedpackage *pkg) {
 		fprintf(stderr,"Adding tracked package '%s'_'%s' to '%s'\n",
 				pkg->sourcename, pkg->sourceversion,
 				t->codename);
+//TODO: find out why I did this here:
 	return RET_OK;
 	trackedpackage_free(pkg);
 	return r;
@@ -598,7 +617,7 @@ retvalue tracking_save(trackingdb t, struct trackedpackage *pkg) {
 	return r;
 }
 
-retvalue tracking_clearall(trackingdb t) {
+retvalue tracking_removeall(trackingdb t) {
 	DBC *cursor;
 	DBT key,data;
 	retvalue r;
@@ -606,7 +625,7 @@ retvalue tracking_clearall(trackingdb t) {
 
 	cursor = NULL;
 	if( (dbret = t->db->cursor(t->db,NULL,&cursor,0)) != 0 ) {
-		t->db->err(t->db, dbret, "tracking_clearall dberror(cursor):");
+		t->db->err(t->db, dbret, "tracking_removeall dberror(cursor):");
 		return RET_DBERR(dbret);
 	}
 	r = RET_OK;
@@ -615,17 +634,17 @@ retvalue tracking_clearall(trackingdb t) {
 	while( (dbret=cursor->c_get(cursor,&key,&data,DB_NEXT)) == 0 ) {
 		dbret = cursor->c_del(cursor,0);
 		if( dbret != 0 ) {
-			t->db->err(t->db, dbret, "tracking_clearall dberror(del):");
+			t->db->err(t->db, dbret, "tracking_removeall dberror(del):");
 			RET_UPDATE(r,RET_DBERR(dbret));
 		}
 	}
 	if( dbret != DB_NOTFOUND ) {
 		(void)cursor->c_close(cursor);
-		t->db->err(t->db, dbret, "tracking_clearall dberror(get):");
+		t->db->err(t->db, dbret, "tracking_removeall dberror(get):");
 		return RET_DBERR(dbret);
 	}
 	if( (dbret=cursor->c_close(cursor)) != 0 ) {
-		t->db->err(t->db, dbret, "tracking_clearall dberror(close):");
+		t->db->err(t->db, dbret, "tracking_removeall dberror(close):");
 		return RET_DBERR(dbret);
 	}
 	return r;
@@ -709,7 +728,7 @@ static void print(const char *codename,const struct trackedpackage *pkg){
 	}
 	fputs("\n",stdout);
 }
-	
+
 retvalue tracking_printall(trackingdb t) {
 	DBC *cursor;
 	DBT key,data;
@@ -744,7 +763,7 @@ retvalue tracking_printall(trackingdb t) {
 	}
 	return r;
 }
-	
+
 retvalue tracking_parse(/*@null@*//*@only@*/char *option,struct distribution *d) {
 	/*@temp@*/char *p,*q;
 
@@ -945,7 +964,7 @@ static inline retvalue trackedpackage_removeall(trackingdb tracks, struct tracke
 	char *id;
 	int i;
 
-	printf("[trackedpackage_removeall %s %s %s]\n",tracks->codename,pkg->sourcename,pkg->sourceversion);
+//	printf("[trackedpackage_removeall %s %s %s]\n",tracks->codename,pkg->sourcename,pkg->sourceversion);
 	id = calc_trackreferee(tracks->codename, pkg->sourcename, pkg->sourceversion);
 	if( id == NULL )
 		return RET_ERROR_OOM;
@@ -1004,7 +1023,7 @@ static inline retvalue trackedpackage_removeunneeded(trackingdb tracks, struct t
 					result = RET_ERROR_OOM;
 			}
 			if( id != NULL ) {
-				printf("[trackedpackage_removeunneeded %s %s %s: '%s']\n",tracks->codename,pkg->sourcename,pkg->sourceversion, filekey);
+//				printf("[trackedpackage_removeunneeded %s %s %s: '%s']\n",tracks->codename,pkg->sourcename,pkg->sourceversion, filekey);
 				r = references_decrement(refs, filekey, id);
 				RET_UPDATE(result,r);
 				r = strlist_add(dereferenced, filekey);
@@ -1042,13 +1061,15 @@ static inline retvalue trackedpackage_tidy(trackingdb tracks, struct trackedpack
 }
 
 retvalue trackingdata_finish(trackingdb tracks, struct trackingdata *d, references refs, struct strlist *dereferenced) {
-	
 	retvalue r;
 	assert( d->tracks == tracks );
-	r = trackedpackage_tidy(tracks, d->pkg, refs, dereferenced);
-	r = tracking_save(tracks, d->pkg);
+	if( d->pkg != NULL ) {
+		r = trackedpackage_tidy(tracks, d->pkg, refs, dereferenced);
+		r = tracking_save(tracks, d->pkg);
+	} else
+		r = RET_OK;
 	d->pkg = NULL;
-	/* call for all rememebered actions... */
+	/* call for all remembered actions... */
 	while( d->remembered != NULL ) {
 		struct trackingdata_remember *h = d->remembered;
 		struct trackedpackage *pkg;
@@ -1066,3 +1087,64 @@ retvalue trackingdata_finish(trackingdb tracks, struct trackingdata *d, referenc
 	return r;
 
 }
+
+retvalue tracking_tidyall(trackingdb t, references refs, struct strlist *dereferenced) {
+	DBC *cursor;
+	DBT key,data;
+	retvalue result,r;
+	int dbret;
+
+	result = RET_NOTHING;
+	cursor = NULL;
+	if( (dbret = t->db->cursor(t->db,NULL,&cursor,0)) != 0 ) {
+		t->db->err(t->db, dbret, "tracking_tidyall dberror(cursor):");
+		return RET_DBERR(dbret);
+	}
+	CLEARDBT(key);
+	CLEARDBT(data);
+	while( (dbret=cursor->c_get(cursor,&key,&data,DB_NEXT)) == 0 ) {
+		struct trackedpackage *pkg;
+		r = parseunknowndata(key, data, &pkg);
+		if( RET_WAS_ERROR(r) )
+			return r;
+		r = trackedpackage_tidy(t, pkg, refs, dereferenced);
+		RET_UPDATE(result, r);
+		if( pkg->flags.deleted ) {
+			/* delete if delete is requested
+			 * (all unreferencing has to be done before) */
+			dbret = cursor->c_del(cursor, 0);
+			if( dbret != 0 ) {
+				trackedpackage_free(pkg);
+				(void)cursor->c_close(cursor);
+				t->db->err(t->db, dbret, "tracking_tidyall(delete) dberror:");
+				return RET_DBERR(dbret);
+			}
+		} else {
+			r = gendata(&data,pkg);
+			if( RET_WAS_ERROR(r) ) {
+				(void)cursor->c_close(cursor);
+				return r;
+			}
+			dbret = cursor->c_put(cursor,&key,&data,DB_CURRENT);
+			free(data.data);
+			if( dbret != 0 ) {
+				trackedpackage_free(pkg);
+				(void)cursor->c_close(cursor);
+				t->db->err(t->db, dbret, "tracking_tidyall c_put:");
+				return RET_DBERR(dbret);
+			}
+		}
+		trackedpackage_free(pkg);
+	}
+	if( dbret != DB_NOTFOUND ) {
+		(void)cursor->c_close(cursor);
+		t->db->err(t->db, dbret, "tracking_tidyall dberror(get):");
+		return RET_DBERR(dbret);
+	}
+	if( (dbret=cursor->c_close(cursor)) != 0 ) {
+		t->db->err(t->db, dbret, "tracking_tidyall dberror(close):");
+		return RET_DBERR(dbret);
+	}
+	return result;
+}
+
