@@ -1,7 +1,7 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2003,2004,2005,2006 Bernhard R. Link
+ *  Copyright (C) 2003,2004,2005,2006,2007 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as 
+ *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
@@ -43,7 +43,9 @@
 #include "terms.h"
 #include "filterlist.h"
 #include "readrelease.h"
+#include "log.h"
 #include "donefile.h"
+#include "freespace.h"
 
 // TODO: what about other signatures? Is hard-coding ".gpg" sensible?
 
@@ -70,7 +72,7 @@ updates_getupstreams instances them for a given distribution:
       |  \ /             |        |
       | u_target -> u_index -> u_index -> NULL
       |   |
-      |  \ /                     
+      |  \ /
       |  NULL              .           .
      \ /                   |           |
    distribution ---> u_origin -> u_origin -> NULL
@@ -269,8 +271,8 @@ inline static retvalue parse_pattern(const char *confdir,const char *chunk, stru
 	struct strlist componentslist,architectureslist;
 	char *formula,*filename;
 	retvalue r;
-	static const char * const allowedfields[] = {"Name", "Method", 
-"Config", "Suite", "Architectures", "Components", "UDebComponents", 
+	static const char * const allowedfields[] = {"Name", "Method",
+"Config", "Suite", "Architectures", "Components", "UDebComponents",
 "IgnoreRelease", "VerifyRelease", "FilterFormula", "FilterList",
 "ListHook", "Fallback", NULL};
 
@@ -338,7 +340,7 @@ inline static retvalue parse_pattern(const char *confdir,const char *chunk, stru
 		update->suite_from = NULL;
 
 	/* * Check which architectures to update from * */
-	r = chunk_getwordlist(chunk,"Architectures",&architectureslist);
+	r = chunk_getuniqwordlist(chunk,"Architectures",&architectureslist);
 	if( RET_WAS_ERROR(r) ) {
 		update_pattern_free(update);
 		return r;
@@ -357,7 +359,7 @@ inline static retvalue parse_pattern(const char *confdir,const char *chunk, stru
 	}
 
 	/* * Check which components to update from * */
-	r = chunk_getwordlist(chunk,"Components",&componentslist);
+	r = chunk_getuniqwordlist(chunk,"Components",&componentslist);
 	if( RET_WAS_ERROR(r) ) {
 		update_pattern_free(update);
 		return r;
@@ -376,7 +378,7 @@ inline static retvalue parse_pattern(const char *confdir,const char *chunk, stru
 	}
 
 	/* * Check which components to update udebs from * */
-	r = chunk_getwordlist(chunk,"UDebComponents",&componentslist);
+	r = chunk_getuniqwordlist(chunk,"UDebComponents",&componentslist);
 	if( RET_WAS_ERROR(r) ) {
 		update_pattern_free(update);
 		return r;
@@ -521,7 +523,7 @@ static retvalue instance_pattern(const char *listdir,
 			}
 		}
 	}
-	
+
 	*origins = update;
 	return RET_OK;
 }
@@ -551,7 +553,7 @@ retvalue updates_getpatterns(const char *confdir,struct update_pattern **pattern
 	retvalue r;
 
 	updatesfile = calc_dirconcat(confdir,"updates");
-	if( updatesfile == NULL ) 
+	if( updatesfile == NULL )
 		return RET_ERROR_OOM;
 	data.patterns = &update;
 	data.confdir = confdir;
@@ -575,7 +577,7 @@ static retvalue getorigins(const char *listdir,const struct update_pattern *patt
 	for( i = 0; i < distribution->updates.count ; i++ ) {
 		const char *name = distribution->updates.values[i];
 		const struct update_pattern *pattern;
-		struct update_origin *update;
+		struct update_origin *update IFSTUPIDCC(=NULL);
 		retvalue r;
 
 		if( strcmp(name,"-") == 0 ) {
@@ -599,6 +601,7 @@ static retvalue getorigins(const char *listdir,const struct update_pattern *patt
 			RET_UPDATE(result,RET_ERROR);
 			break;
 		}
+		IFSTUPIDCC(update = NULL;)
 
 		r = instance_pattern(listdir,pattern,distribution,&update);
 		RET_UPDATE(result,r);
@@ -767,7 +770,7 @@ static inline retvalue findmissingupdate(int count,const struct distribution *di
 			struct update_origin *u;
 
 			u = updates;
-			while( u != NULL && strcmp(u->pattern->name,update) != 0 ) 
+			while( u != NULL && strcmp(u->pattern->name,update) != 0 )
 				u = u->next;
 			if( u == NULL ) {
 				fprintf(stderr,"Update '%s' is listed in distribution '%s', but was not found!\n",update,distribution->codename);
@@ -792,7 +795,7 @@ retvalue updates_calcindices(const char *listdir,const struct update_pattern *pa
 	u_ds = NULL;
 	r = RET_OK;
 
-	for( distribution = distributions ; distribution != NULL ; 
+	for( distribution = distributions ; distribution != NULL ;
 			       distribution = distribution->next ) {
 		struct update_distribution *u_d;
 
@@ -943,7 +946,7 @@ retvalue updates_clearlists(const char *listdir,struct update_distribution *dist
 		dir = opendir(listdir);
 		if( dir == NULL ) {
 			int e = errno;
-			fprintf(stderr,"Error opening directory '%s' (error %d=%m)!",listdir,e);
+			fprintf(stderr,"Error opening directory '%s' (error %d=%m)!\n",listdir,e);
 			free(pattern);
 			return RET_ERRNO(e);
 		}
@@ -984,7 +987,7 @@ static retvalue updates_startup(struct aptmethodrun *run,struct update_distribut
 
 	result = RET_NOTHING;
 	for( d=distributions ; d != NULL ; d=d->next) {
-		if( d->distribution->deb_override != NULL || 
+		if( d->distribution->deb_override != NULL ||
 		    d->distribution->dsc_override != NULL ||
 		    d->distribution->udeb_override != NULL ) {
 			if( verbose >= 0 )
@@ -1051,6 +1054,12 @@ static inline retvalue readchecksums(struct update_origin *origin) {
 		r = signature_check(origin->pattern->verifyrelease,
 				origin->releasegpgfile,
 				origin->releasefile);
+		if( r == RET_NOTHING ) {
+			fprintf(stderr, "Error: No accepted signature found for update rule %s for distribution %s!\n",
+					origin->pattern->name,
+					origin->distribution->codename);
+			r = RET_ERROR_BADSIG;
+		}
 		if( RET_WAS_ERROR(r) ) {
 			origin->failed = TRUE;
 			return r;
@@ -1110,7 +1119,7 @@ static inline retvalue queueindex(struct update_index *index) {
 			}
 			return r;
 		}
-		
+
 	}
 	fprintf(stderr,"Could not find '%s' within the Releasefile of '%s':\n'%s'\n",index->upstream,origin->pattern->name,origin->releasefile);
 	return RET_ERROR_WRONG_MD5;
@@ -1185,7 +1194,7 @@ static retvalue updates_queuelists(struct update_distribution *distributions,boo
 static retvalue calllisthook(const char *listhook,struct update_index *index) {
 	char *newfilename;
 	pid_t f,c;
-	int status; 
+	int status;
 
 	newfilename = mprintf("%s_changed",index->filename);
 	if( newfilename == NULL )
@@ -1250,7 +1259,7 @@ static retvalue updates_calllisthooks(struct update_distribution *distributions)
 		for( target=d->targets; target!=NULL ; target=target->next ) {
 			if( target->nothingnew )
 				continue;
-			for( index=target->indices ; index != NULL ; 
+			for( index=target->indices ; index != NULL ;
 						     index=index->next ) {
 				if( index->origin == NULL )
 					continue;
@@ -1301,18 +1310,18 @@ static upgrade_decision ud_decide_by_pattern(void *privdata, const char *package
 	return UD_UPGRADE;
 }
 
-static inline retvalue searchformissing(const char *dbdir,struct update_target *u) {
+static inline retvalue searchformissing(FILE *out,const char *dbdir,struct update_target *u) {
 	struct update_index *index;
 	retvalue result,r;
 
 	if( u->nothingnew ) {
 		if( verbose >= 0 ) {
-		fprintf(stderr,"  nothing new for '%s' (use --noskipold to process anyway)\n",u->target->identifier);
+		fprintf(out,"  nothing new for '%s' (use --noskipold to process anyway)\n",u->target->identifier);
 		}
 		return RET_NOTHING;
 	}
 	if( verbose > 2 )
-		fprintf(stderr,"  processing updates for '%s'\n",u->target->identifier);
+		fprintf(out,"  processing updates for '%s'\n",u->target->identifier);
 	r = upgradelist_initialize(&u->upgradelist,u->target,dbdir);
 	if( RET_WAS_ERROR(r) )
 		return r;
@@ -1323,7 +1332,7 @@ static inline retvalue searchformissing(const char *dbdir,struct update_target *
 
 		if( index->origin == NULL ) {
 			if( verbose > 4 )
-				fprintf(stderr,"  marking everything to be deleted\n");
+				fprintf(out,"  marking everything to be deleted\n");
 			r = upgradelist_deleteall(u->upgradelist);
 			if( RET_WAS_ERROR(r) )
 				u->incomplete = TRUE;
@@ -1343,7 +1352,7 @@ static inline retvalue searchformissing(const char *dbdir,struct update_target *
 		}
 
 		if( verbose > 4 )
-			fprintf(stderr,"  reading '%s'\n",index->filename);
+			fprintf(out,"  reading '%s'\n",index->filename);
 		assert(index->origin->download!= NULL);
 		r = upgradelist_update(u->upgradelist,
 				index->origin->download,index->filename,
@@ -1357,17 +1366,17 @@ static inline retvalue searchformissing(const char *dbdir,struct update_target *
 		if( RET_WAS_ERROR(r) )
 			return result;
 	}
-	
+
 	return result;
 }
 
-static retvalue updates_readindices(const char *dbdir,struct update_distribution *d) {
+static retvalue updates_readindices(FILE *out,const char *dbdir,struct update_distribution *d) {
 	retvalue result,r;
 	struct update_target *u;
 
 	result = RET_NOTHING;
 	for( u=d->targets ; u != NULL ; u=u->next ) {
-		r = searchformissing(dbdir,u);
+		r = searchformissing(out, dbdir, u);
 		if( RET_WAS_ERROR(r) )
 			u->incomplete = TRUE;
 		RET_UPDATE(result,r);
@@ -1400,11 +1409,16 @@ static retvalue updates_install(const char *dbdir,filesdb filesdb,references ref
 	retvalue result,r;
 	struct update_target *u;
 
+	assert( logger_isprepared(distribution->distribution->logger) );
+
 	result = RET_NOTHING;
 	for( u=distribution->targets ; u != NULL ; u=u->next ) {
 		if( u->nothingnew )
 			continue;
-		r = upgradelist_install(u->upgradelist,dbdir,filesdb,refs,u->ignoredelete,dereferencedfilekeys);
+		r = upgradelist_install(u->upgradelist,
+				distribution->distribution->logger,
+				dbdir, filesdb, refs,
+				u->ignoredelete, dereferencedfilekeys);
 		RET_UPDATE(distribution->distribution->status, r);
 		if( RET_WAS_ERROR(r) )
 			u->incomplete = TRUE;
@@ -1489,12 +1503,17 @@ static retvalue updates_downloadlists(const char *methoddir,struct aptmethodrun
 	return result;
 }
 
-retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *distributions,bool_t nolistsdownload,bool_t skipold,struct strlist *dereferencedfilekeys) {
+retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *distributions,bool_t nolistsdownload,bool_t skipold,struct strlist *dereferencedfilekeys,enum spacecheckmode mode,off_t reserveddb,off_t reservedother) {
 	retvalue result,r;
 	struct update_distribution *d;
 	struct aptmethodrun *run;
 	struct downloadcache *cache;
 
+	for( d=distributions ; d != NULL ; d=d->next) {
+		r = distribution_prepareforwriting(d->distribution);
+		if( RET_WAS_ERROR(r) )
+			return r;
+	}
 	r = aptmethod_initialize_run(&run);
 	if( RET_WAS_ERROR(r) )
 		return r;
@@ -1524,11 +1543,12 @@ retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,
 			aptmethod_shutdown(run);
 			return result;
 		}
-		/* TODO: 
+		/* TODO:
 		 * add a check if some of the upstreams without Release files
 		 * are unchanged and if this changes anything? */
 		if( !anythingtodo ) {
-			fprintf(stderr,"Nothing to do found. (Use --noskipold to force processing)\n");
+			if( verbose >= 0 )
+				fprintf(stderr,"Nothing to do found. (Use --noskipold to force processing)\n");
 			aptmethod_shutdown(run);
 			if( RET_IS_OK(result) )
 				return RET_NOTHING;
@@ -1548,16 +1568,16 @@ retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,
 
 	/* Then get all packages */
 	if( verbose >= 0 )
-		fprintf(stderr,"Calculating packages to get...\n");
-	r = downloadcache_initialize(&cache);
+		printf("Calculating packages to get...\n");
+	r = downloadcache_initialize(dbdir, mode, reserveddb, reservedother, &cache);
 	if( !RET_IS_OK(r) ) {
 		aptmethod_shutdown(run);
 		RET_UPDATE(result,r);
 		return result;
 	}
-	
+
 	for( d=distributions ; d != NULL ; d=d->next) {
-		r = updates_readindices(dbdir,d);
+		r = updates_readindices(stdout, dbdir, d);
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(r) )
 			break;
@@ -1566,6 +1586,11 @@ retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,
 		if( RET_WAS_ERROR(r) )
 			break;
 	}
+	if( !RET_WAS_ERROR(result) ) {
+		r = space_check(cache->devices);
+		RET_ENDUPDATE(result,r);
+	}
+
 	if( RET_WAS_ERROR(result) ) {
 		for( d=distributions ; d != NULL ; d=d->next) {
 			struct update_target *u;
@@ -1580,15 +1605,13 @@ retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,
 		return result;
 	}
 	if( verbose >= 0 )
-		fprintf(stderr,"Getting packages...\n");
+		printf("Getting packages...\n");
+	r = downloadcache_free(cache);
+	RET_ENDUPDATE(result,r);
 	r = aptmethod_download(run,methoddir,filesdb);
 	RET_UPDATE(result,r);
 	if( verbose > 0 )
-		fprintf(stderr,"Freeing some memory...\n");
-	r = downloadcache_free(cache);
-	RET_UPDATE(result,r);
-	if( verbose > 0 )
-		fprintf(stderr,"Shutting down aptmethods...\n");
+		printf("Shutting down aptmethods...\n");
 	r = aptmethod_shutdown(run);
 	RET_UPDATE(result,r);
 
@@ -1603,7 +1626,7 @@ retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,
 		return result;
 	}
 	if( verbose >= 0 )
-		fprintf(stderr,"Installing (and possibly deleting) packages...\n");
+		printf("Installing (and possibly deleting) packages...\n");
 
 	for( d=distributions ; d != NULL ; d=d->next) {
 		r = updates_install(dbdir,filesdb,refs,d,dereferencedfilekeys);
@@ -1619,6 +1642,7 @@ retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,
 			markdone(u);
 		}
 	}
+	logger_wait();
 
 	return result;
 }
@@ -1679,9 +1703,9 @@ retvalue updates_checkupdate(const char *dbdir,const char *methoddir,struct upda
 	/* Then look what packages to get */
 	if( verbose >= 0 )
 		fprintf(stderr,"Calculating packages to get...\n");
-	
+
 	for( d=distributions ; d != NULL ; d=d->next) {
-		r = updates_readindices(dbdir,d);
+		r = updates_readindices(stderr, dbdir, d);
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(r) )
 			break;
@@ -1696,6 +1720,11 @@ retvalue updates_predelete(const char *dbdir,const char *methoddir,references re
 	struct update_distribution *d;
 	struct aptmethodrun *run;
 
+	for( d=distributions ; d != NULL ; d=d->next) {
+		r = distribution_prepareforwriting(d->distribution);
+		if( RET_WAS_ERROR(r) )
+			return r;
+	}
 	r = aptmethod_initialize_run(&run);
 	if( RET_WAS_ERROR(r) )
 		return r;
@@ -1725,11 +1754,12 @@ retvalue updates_predelete(const char *dbdir,const char *methoddir,references re
 			aptmethod_shutdown(run);
 			return result;
 		}
-		/* TODO: 
+		/* TODO:
 		 * add a check if some of the upstreams without Release files
 		 * are unchanged and if this changes anything? */
 		if( !anythingtodo ) {
-			fprintf(stderr,"Nothing to do found. (Use --noskipold to force processing)\n");
+			if( verbose >= 0 )
+				printf("Nothing to do found. (Use --noskipold to force processing)\n");
 			aptmethod_shutdown(run);
 			if( RET_IS_OK(result) )
 				return RET_NOTHING;
@@ -1748,21 +1778,21 @@ retvalue updates_predelete(const char *dbdir,const char *methoddir,references re
 	}
 
 	if( verbose > 0 )
-		fprintf(stderr,"Shutting down aptmethods...\n");
+		printf("Shutting down aptmethods...\n");
 
 	r = aptmethod_shutdown(run);
 	RET_UPDATE(result,r);
 	if( RET_WAS_ERROR(result) ) {
 		return result;
 	}
-	
+
 	if( verbose >= 0 )
-		fprintf(stderr,"Removing obsolete or to be replaced packages...\n");
+		printf("Removing obsolete or to be replaced packages...\n");
 	for( d=distributions ; d != NULL ; d=d->next) {
 		struct update_target *u;
 
 		for( u=d->targets ; u != NULL ; u=u->next ) {
-			r = searchformissing(dbdir,u);
+			r = searchformissing(stdout, dbdir, u);
 			RET_UPDATE(result,r);
 			if( RET_WAS_ERROR(r) ) {
 				u->incomplete = TRUE;
@@ -1773,7 +1803,9 @@ retvalue updates_predelete(const char *dbdir,const char *methoddir,references re
 				u->upgradelist = NULL;
 				continue;
 			}
-			r = upgradelist_predelete(u->upgradelist,dbdir,refs,dereferencedfilekeys);
+			r = upgradelist_predelete(u->upgradelist,
+					d->distribution->logger,
+					dbdir, refs, dereferencedfilekeys);
 			RET_UPDATE(d->distribution->status, r);
 			if( RET_WAS_ERROR(r) )
 				u->incomplete = TRUE;
@@ -1784,16 +1816,20 @@ retvalue updates_predelete(const char *dbdir,const char *methoddir,references re
 				return r;
 		}
 	}
+	logger_wait();
 	return result;
 }
 
-static retvalue singledistributionupdate(const char *dbdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *d,bool_t nolistsdownload,bool_t skipold, struct strlist *dereferencedfilekeys) {
+static retvalue singledistributionupdate(const char *dbdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *d,bool_t nolistsdownload,bool_t skipold, struct strlist *dereferencedfilekeys,enum spacecheckmode mode,off_t reserveddb,off_t reservedother) {
 	struct aptmethodrun *run;
 	struct downloadcache *cache;
 	struct update_origin *origin;
 	struct update_target *target;
 	retvalue result,r;
 
+	result = distribution_prepareforwriting(d->distribution);
+	if( RET_WAS_ERROR(result) )
+		return result;
 	result = RET_OK;
 
 	r = aptmethod_initialize_run(&run);
@@ -1891,9 +1927,9 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 		}
 		/* Then get all packages */
 		if( verbose >= 0 )
-			fprintf(stderr,"Calculating packages to get for %s's %s...\n",d->distribution->codename,target->target->identifier);
-		r = downloadcache_initialize(&cache);
-		RET_UPDATE(result,r);	
+			printf("Calculating packages to get for %s's %s...\n",d->distribution->codename,target->target->identifier);
+		r = downloadcache_initialize(dbdir, mode, reserveddb, reservedother, &cache);
+		RET_UPDATE(result,r);
 		if( !RET_IS_OK(r) ) {
 			(void)downloadcache_free(cache);
 			if( RET_WAS_ERROR(r) ) {
@@ -1902,7 +1938,7 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 			}
 			continue;
 		}
-		r = searchformissing(dbdir,target);
+		r = searchformissing(stdout, dbdir, target);
 		if( RET_WAS_ERROR(r) )
 			target->incomplete = TRUE;
 		else if( r == RET_NOTHING ) {
@@ -1938,7 +1974,11 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 
 		if( verbose >= 0 )
 			fprintf(stderr,"Installing/removing packages for %s's %s...\n",d->distribution->codename,target->target->identifier);
-		r = upgradelist_install(target->upgradelist,dbdir,filesdb,refs,target->ignoredelete,dereferencedfilekeys);
+		r = upgradelist_install(target->upgradelist,
+				d->distribution->logger,
+				dbdir, filesdb, refs,
+				target->ignoredelete, dereferencedfilekeys);
+		logger_wait();
 		if( RET_WAS_ERROR(r) )
 			target->incomplete = TRUE;
 		RET_UPDATE(result,r);
@@ -1958,7 +1998,7 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 	return result;
 }
 
-retvalue updates_iteratedupdate(const char *confdir,const char *dbdir,const char *distdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *distributions,bool_t nolistsdownload,bool_t skipold,struct strlist *dereferencedfilekeys, enum exportwhen export) {
+retvalue updates_iteratedupdate(const char *confdir,const char *dbdir,const char *distdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *distributions,bool_t nolistsdownload,bool_t skipold,struct strlist *dereferencedfilekeys, enum exportwhen export,enum spacecheckmode mode,off_t reserveddb,off_t reservedother) {
 	retvalue result,r;
 	struct update_distribution *d;
 
@@ -1973,7 +2013,7 @@ retvalue updates_iteratedupdate(const char *confdir,const char *dbdir,const char
 
 	result = RET_NOTHING;
 	for( d=distributions ; d != NULL ; d=d->next) {
-		if( d->distribution->deb_override != NULL || 
+		if( d->distribution->deb_override != NULL ||
 		    d->distribution->dsc_override != NULL ||
 		    d->distribution->udeb_override!= NULL ) {
 			if( verbose >= 0 )
@@ -1982,7 +2022,7 @@ retvalue updates_iteratedupdate(const char *confdir,const char *dbdir,const char
 		if( d->distribution->tracking != dt_NONE ) {
 			fprintf(stderr,"WARNING: Updating does not update trackingdata. Trackingdata of %s will be outdated!\n",d->distribution->codename);
 		}
-		r = singledistributionupdate(dbdir,methoddir,filesdb,refs,d,nolistsdownload,skipold,dereferencedfilekeys);
+		r = singledistributionupdate(dbdir,methoddir,filesdb,refs,d,nolistsdownload,skipold,dereferencedfilekeys, mode, reserveddb, reservedother);
 		RET_ENDUPDATE(d->distribution->status,r);
 		RET_UPDATE(result,r);
 		r = distribution_export(export,d->distribution,confdir,dbdir,distdir,filesdb);
